<?php
// Exit if accessed directly
if (!defined('ABSPATH')) {
	exit();
}
use Wovax\IDX\Admin\Pages\SearchAppearanceAdmin;

$tabs     = array('initial-setup', 'users', 'styling', 'search-appearance');
$sel_tab  = isset($_GET['tab']) ? strtolower(trim($_GET['tab'])) : '';
if(in_array($sel_tab, $tabs, TRUE) === FALSE) {
    $sel_tab = 'initial-setup';
}
?>
<div class="wrap">
  <h1>Wovax IDX Settings</h1>
  <h2 class="nav-tab-wrapper">
<?php
    foreach($tabs as $tab) {
        $html  = "<a href=\"?page=wovax_idx_settings&tab=$tab\" ";
        $html .= 'class="nav-tab';
        if($tab === $sel_tab) {
            $html .= ' nav-tab-active';
        }
        $html .= '">'.ucwords(str_replace('-', ' ', $tab))."</a>\n";
        echo $html;
    }
?></h2>
<?php

switch($sel_tab) {
    case 'users':
        wovax_idx_display_users();
        break;
    case 'styling':
        wovax_idx_display_styling();
        break;
    case 'search-appearance':
        (new SearchAppearanceAdmin())->renderPage();
        break;
    default:
        wovax_idx_display_initial_setup();
        break;
}

?>
  </form>
</div>
  <?php
/* Display HTML Initial Setup Tab */

function wovax_idx_display_initial_setup() {
?>
  <form method="post" action="options.php">
  <?php
  settings_fields("settings_menu_initial_setup");
  do_settings_sections("settings_menu_initial_setup");
?>
  <section id="initial_setup"> <!-- Initial Setup tab content -->
    <table class="form-table">
      <tbody>
        <tr>
          <th scope="row">
            <label for="email">Email</label>
          </th>
          <td>
            <input name="wovax-idx-settings-webmaster-email" type="email" class="regular-text" id="email" value="<?php echo esc_attr(get_option('wovax-idx-settings-webmaster-email')); ?>">
            <p class="description" id="tagline-description">Must be a valid email address for IDX feed setup.</p>
          </td>
        </tr>
        <tr>
          <th scope="row">
            <label>Environment</label>
          </th>
          <td>
            <fieldset>
              <label for="environment">
                <input name="wovax-idx-settings-environment" type="radio" id="development" value="development" <?php checked('development', get_option('wovax-idx-settings-environment')); ?> > Development (sample MLS data)
              </label>
              <br />
              <label for="production">
                <input name="wovax-idx-settings-environment" type="radio" id="production" value="production" <?php checked('production', get_option('wovax-idx-settings-environment')); ?> > Production (live MLS data)
              </label>
              <p class="description" id="tagline-description">Production requires an active <a href="https://wovax.com/wordpress-idx/" title="Wovax IDX" target="_blank">Wovax IDX</a> subscription.</p>
            </fieldset>
          </td>
        </tr>
        <tr>
          <th scope="row">
            <label for="wovax-idx-settings-search-results-page">Search Results Page ID</label>
          </th>
          <td>
            <input type="text" name="wovax-idx-settings-search-results-page" id="wovax-idx-settings-search-results-page" placeholder="ID Search Results Page" value="<?php echo esc_attr(get_option('wovax-idx-settings-search-results-page')); ?>" />
            <input type="button" id="wovax-idx-settings-search-results-button" class="button button-secondary" value="Create Search Results Page">
            <div id="wovax_idx_message" class="wovax-idx-settings-color-green wovax-idx-settings-display-none">The page was created.</div>
            <p class="description" id="tagline-description">Creates a page called Search Results containing the [wovax-idx-search-results] shortcode.</p>
          </td>
        </tr>
        <tr>
          <th scope="row">
            <label for="wovax-idx-settings-listing-details-page">Listing Details Page ID</label>
          </th>
          <td>
            <input type="text" name="wovax-idx-settings-listing-details-page" id="wovax-idx-settings-listing-details-page" placeholder="ID Listing Details Page" value="<?php echo esc_attr(get_option('wovax-idx-settings-listing-details-page')); ?>" />
            <input type="button" id="wovax-idx-settings-listing-details-button" class="button button-secondary" value="Create Listing Details Page">
            <div id="wovax_idx_message_listing" class="wovax-idx-settings-color-green wovax-idx-settings-display-none">The page was created.</div>
            <p class="description" id="tagline-description">Creates a page called Listing Details containing the [wovax-idx-listing-details] shortcode.</p>
          </td>
        </tr>
        <tr>
          <th scope="row">
            <label for="google_maps_api_key">Google Maps API Key ID</label>
          </th>
          <td>
            <input name="wovax-idx-settings-google-maps-api-key" type="text" class="regular-text" id="google_maps_api_key" value="<?php echo esc_attr(get_option('wovax-idx-settings-google-maps-api-key')); ?>">
            <p class="description" id="tagline-description">Maps requires an active <a href="https://developers.google.com/maps/documentation/javascript/get-api-key" title="Google Maps API key" target="_blank">Google Maps API key</a>.</p>
          </td>
        </tr>
        <tr>
        </tbody>
      </table>
        <?php submit_button(); ?>
    </section>
    <?php
}

/* Display HTML User Tab */
function wovax_idx_display_users() {
?>
    <form method="post" action="options.php">
    <?php
  settings_fields("settings_menu_users");
  do_settings_sections("settings_menu_users");
?>
    <section id="users"> <!-- Users tab content -->
      <table class="form-table">
        <tbody>
          <tr>
            <th scope="row">User Registration</th>
            <td>
              <fieldset>
                <label for="wovax-idx-settings-users-registration">
                  <input type="checkbox" name="wovax-idx-settings-users-registration" value="yes" <?php checked('yes', get_option('wovax-idx-settings-users-registration')); ?> > Allow visitors to create a user account.
                </label>
                <br />
              </fieldset>
            </td>
          </tr>
          <tr>
            <th scope="row">Force User Registration</th>
            <td>
              <fieldset>
                <label for="wovax-idx-settings-users-registration-force">
                  <input type="checkbox" name="wovax-idx-settings-users-registration-force" value="yes"  id="wovax-idx-settings-users-registration-force" <?php checked('yes', get_option('wovax-idx-settings-users-registration-force')); ?>> Force visitors to create a user account.
                </label>

                <br />
                <label for="wovax-idx-settings-users-registration-force-count"><input name="wovax-idx-settings-users-registration-force-count"" type="number" min="0" step="1" id="wovax-idx-settings-users-registration-force-count" value="<?php echo esc_attr(get_option('wovax-idx-settings-users-registration-force-count')); ?>" class="small-text"> Listings vistors can view before being forced to register a user account.</label>
              </fieldset>
            </td>
          </tr>
          <tr>
            <th scope="row">Favorites</th>
            <td>
              <fieldset>
                <label for="wovax-idx-settings-users-favorites">
                  <input type="checkbox" name="wovax-idx-settings-users-favorites" value="yes" <?php checked('yes', get_option('wovax-idx-settings-users-favorites')); ?>> Allow users to favorite properties.
                </label>
              </fieldset>
            </td>
          </tr>
          <tr>
            <th scope="row">Saved Searches</th>
            <td>
              <fieldset>
                <label for="wovax-idx-settings-users-saved-searches">
                  <input type="checkbox" name="wovax-idx-settings-users-saved-searches" value="yes" <?php checked('yes', get_option('wovax-idx-settings-users-saved-searches')); ?>> Allow users to save searches.
                </label>
              </fieldset>
            </td>
          </tr>
        </tbody>
      </table>
      <?php submit_button(); ?>
    </section>

    <?php
  }

/* Display HTML Styling Tab */
function wovax_idx_display_styling() {
?>
    <form method="post" action="options.php">
    <?php
  settings_fields("settings_menu_listing");
  do_settings_sections("settings_menu_listing");
?>
    <section id="styling"> <!-- Listings tab content -->
      <table class="form-table">
        <tbody>
          <tr>
            <th scope="row">Include Wovax IDX CSS</th>
            <td>
              <fieldset>
                <label for="include_css">
                  <input type="checkbox" name="wovax-idx-settings-styling-css" value="yes" value="yes" <?php checked('yes', get_option('wovax-idx-settings-styling-css')); ?> > Disable only if you're including your own custom Wovax IDX CSS in a theme, plugin, etc...
                </label>
                <br />
              </fieldset>
            </td>
          </tr>
          <tr>
            <th scope="row">Include Wovax IDX JS</th>
            <td>
              <fieldset>
                <label for="include_js">
                  <input type="checkbox" name="wovax-idx-settings-styling-js" value="yes" value="yes" <?php checked('yes', get_option('wovax-idx-settings-styling-js')); ?> > Disable only if you're including your own custom Wovax IDX JS in a theme, plugin, etc...
                </label>
                <br />
              </fieldset>
            </td>
          </tr>
          <tr>
            <th scope="row">Include Wovax IDX Floating Labels</th>
            <td>
              <fieldset>
                <label for="include_js">
                  <input type="checkbox" name="wovax-idx-settings-styling-floating-label" value="yes" value="yes" <?php checked('yes', get_option('wovax-idx-settings-styling-floating-label')); ?> > Disable only if you're including your own custom Wovax IDX Floating Label in a theme, plugin, etc...
                </label>
                <br />
              </fieldset>
            </td>
          </tr>
        </tbody>
      </table>
      <?php submit_button(); ?>
    </section>
    <?php
  }
?>
</form>
