<?php
namespace Wovax\IDX\API;
if (defined('ABSPATH') === FALSE) {
    exit();// Exit via direct access.
}
use WP_Http;
use Wovax\IDX\Settings\IntialSetup;

class WovaxConnect {
    const TRANS_AUTH_TKN = 'wovax-idx-api-auth-tkn';
    const PROTO          = 'https';
    const BASE           = 'connect.wovax.com';
    const VERSION        = 'api';
    private $http        = NULL;
    private $email       = '';
    private $environ     = 'development';
    // Public methods
    public function __construct($email, $production = FALSE) {
        $this->http = new WP_Http();
        $this->email = $email;
        if($production) {
            $this->setToProduction();
        }
    }
    public function getListingDetails($class_id, $prop_id, $fields, $photos = 'all') {
        $data = $this->postRequest(
            'resource_detail',
            array(
                'class_id'     => $class_id,
                'resource_id'  => $prop_id,
                'count_photos' => $photos,
                'data_values'  => json_encode(array_values($fields))
            )
        );
        if(is_array($data) && strlen(trim($data['Street Address'])) < 1) {
            $data['Street Address'] = 'No Street Address Provided';
        }
        return $data;
    }
    public function setToDevelopment() {
        $this->eniron = 'development';
    }
    public function setToProduction() {
        $this->eniron = 'production';
    }
    // Private Methods
    private function getBaseURL() {
        return self::PROTO.'://'.self::BASE.'/'.self::VERSION.'/';
    }
    private function getToken() {
        $val = get_transient(self::TRANS_AUTH_TKN);
        // If token has no expired return it.
        if($val !== FALSE) {
            return $val;
        }
        $response = $this->http->request(
            $this->getBaseURL().'validate_domain',
            array(
                'method'  => "POST",
                'timeout' => 45,
                'body'    => array(
                    'guest_email' => $this->email,
                    'environment' => $this->environ
                )
            )
        );
		if(is_wp_error($response)) {
			return '';
		}
        $body = json_decode($response['body'], TRUE);
        if(!is_array($body)) {
            return '';
        }
        $token  = $body['data'];
        $exp    = intval($body['token_exp']) - time() - 1;
        if($exp < 1) {
            // Try to use it anyways but don't store it.
            return $token;
        }
        // Update the stored token with the new one.
        set_transient(
            self::TRANS_AUTH_TKN,
            $token,
            $exp
        );
		return $token;
    }
    private function postRequest($func, $body) {
        $response = $this->http->request(
            $this->getBaseURL().$func,
            array(
                'method'  => "POST",
                'timeout' => 45,
                'headers' => array("Authorization" => $this->getToken()),
                'body'    => $body
            )
        );
        if(is_wp_error($response)) {
            return 'error';
        }
        $data = json_decode($response['body'], TRUE)['data'];
        if(empty($data)) {
            return 'NO DATA - Data element is empty';
        }
        return $data;
    }
    // Public static methods
    public static function createFromOptions() {
        $opts = new IntialSetup();
        if(strlen($opts->email()) < 1) {
            return NULL;
        }
        $prod = ($opts->mode() == 'production');
        $api = new WovaxConnect($opts->email(), $prod);
        return $api;
    }
}