<?php
namespace Wovax\IDX\Settings;
if (defined('ABSPATH') === FALSE) {
    exit();// Exit via direct access.
}

abstract class Options {
    private $prefix = 'wovax-idx-settings-';
    private $options = array();
    // usefull for plugin uninstallation
    public function deleteAll() {
        foreach($this->options as $name => $info) {
            delete_option($this->prefix.$name);
        }
    }
    protected function add($name, $default) {
        if(preg_match('/^[a-z][0-9a-z\-_]*$/', $name) !== 1) {
            throw new exception('Can not add this option improperly formatted name!');
        }
        $this->options[$name] = array(
            'default' => $default
        );
    }
    protected function addToPrefix($prefix) {
        if(preg_match('/^[a-z][0-9a-z\-_]*$/', $prefix) !== 1) {
            throw new exception('Bad addtional prefix!');
        }
        $this->prefix .= $prefix.'-';
    }
    protected function get($name) {
        if(!array_key_exists($name, $this->options)) {
            throw Exception('To get the option it must first be added!');
        }
        $val = get_option($this->prefix.$name, FALSE);
        if($val === FALSE) {
            $val = $this->options[$name]['default'];
        }
        return $val;
    }
    protected function set($name, $value) {
        if(!array_key_exists($name, $this->options)) {
            throw Exception('To set the option it must first be added!');
        }
        update_option($this->prefix.$name, $value);
    }
}