<?php

if (!defined('ABSPATH')) exit;

// SHORTCODE * by ID
function wovax_idx_get_shortcode_by_id($id) {
  $id = filter_var($id, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id) && filter_var($id, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT * FROM `{$wpdb->base_prefix}wovax_idx_shortcode` WHERE `id` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id));
    return $results;
  }
  else {
    return array();
  }
}

// FILTER * by SHORTCODE ID
function wovax_idx_get_filter_by_shortcode_id($id) {
  $id = filter_var($id, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id) && filter_var($id, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT * FROM `{$wpdb->base_prefix}wovax_idx_shortcode_filters` WHERE `id_shortcode` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id));
    return $results;
  }
  else {
    return array();
  }
}

// RULES * by SHORTCODE ID
function wovax_idx_get_rule_by_shortcode_id($id) {
  $id = filter_var($id, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id) && filter_var($id, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT * FROM `{$wpdb->base_prefix}wovax_idx_shortcode_rules` WHERE `id_shortcode` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id));
    return $results;
  }
  else {
    return array();
  }
}

// ALL ID_FEEDS FROM FEEDS TABLE
function wovax_idx_get_all_id_feed() {
  global $wpdb;
  $sql = "SELECT DISTINCT `id_feed` FROM `{$wpdb->base_prefix}wovax_idx_feed_fields`;";
  $results = $wpdb->get_results($sql);
  return $results;
}

function wovax_idx_get_id_field_by_id_filter($id) {
  $id = filter_var($id, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id) && filter_var($id, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT `id_field` FROM `{$wpdb->base_prefix}wovax_idx_shortcode_filters` WHERE `id_filter` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id));
    return $results;
  }
  else {
    return array();
  }
}

function wovax_idx_get_id_field_by_id_rule($id) {
  $id = filter_var($id, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id) && filter_var($id, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT `id_field` FROM `{$wpdb->base_prefix}wovax_idx_shortcode_rules` WHERE `id_rule` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id));
    return $results;
  }
  else {
    return array();
  }
}

function wovax_idx_get_all_id_fields($id) {
  $id = filter_var($id, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id) && filter_var($id, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT f.`id_field` FROM `{$wpdb->base_prefix}wovax_idx_feed_fields` AS `f` WHERE f.`id_feed` = %d ORDER BY f.`order`;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id));
    return $results;
  }
  else {
    return array();
  }
}

function wovax_idx_get_id_feed_by_id_field($id_field) {
  $id_field = filter_var($id_field, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id_field) && filter_var($id_field, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT `id_feed`, `default_alias` FROM `{$wpdb->base_prefix}wovax_idx_feed_fields` WHERE `id_field` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id_field));
    return $results;
  }
  else {
    return array();
  }
}

function wovax_idx_get_filter_type_by_id_filter($id) {
  $id = filter_var($id, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id) && filter_var($id, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT `filter_type` FROM `{$wpdb->base_prefix}wovax_idx_shortcode_filters` WHERE `id_filter` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id));
    return $results;
  }
  else {
    return array();
  }
}

function wovax_idx_get_feed_attr_by_id_feed($id) {
  $id = filter_var($id, FILTER_SANITIZE_NUMBER_INT);
  if (!empty($id) && filter_var($id, FILTER_VALIDATE_INT)) {
    global $wpdb;
    $sql = "SELECT `attributes` FROM `{$wpdb->base_prefix}wovax_idx_feeds` WHERE `id_feed` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id));
    return $results;
  }
  else {
    return array();
  }
}

function wovax_idx_get_permalink_page_search_result() {
  $id_page = get_option('wovax-idx-settings-search-results-page');
  if ($id_page) {
    return get_permalink($id_page);
  }
  return '';
}

function wovax_idx_get_list_buildings_by_filters($attr, $filters, $useless_var, $view_option, $api_key, $feed_rules) {
  // $filters     = filter_var_array( $filters, FILTER_SANITIZE_STRING );
  $useless_var = filter_var($useless_var, FILTER_SANITIZE_STRING);
  /* $feeds_array = these feeds were delimited on shortcode admin feed tab */
  $feeds_array = json_encode(wovax_idx_return_id_feeds_array($attr[0]->feeds));
  /* $pagination = this was delimited on shortcode admin general tab */
  $pagination = $attr[0]->pagination;
  $per_page = $attr[0]->per_page;
  if (!empty($pagination) && !empty($per_page)) {
    
    // Pagination ON
    $paged = (get_query_var('page')) ? get_query_var('page') : 1;
  }

  // ================================ get id_field and value for every id_filter
  // *All id_feed in table FEEDS
  $id_feeds = wovax_idx_get_feeds_ids_by_shortcode($attr[0]->id);
  $where_params = [];

  // Loop to get and compare every id_feed from array
  $arr = [];
  if ($filters != '') {

    // Loop to get every id_field and push to the array
    foreach($filters as $key => $item) {
      $text = str_replace(' ', '_', $item->field);
      $filter = wovax_idx_get_id_field_by_id_filter($item->id_filter);
      $name = strtolower($item->filter_type . '-' . $text . '-' . $item->id_filter);
      $filter = explode(",", $filter[0]->id_field);
      // filter´s id_feed
      $id_feed_compare = wovax_idx_get_id_feed_by_id_field($filter[0]);
      // values we´ll add to the array 'fields_value'
      $val = filter_var($_GET[$name], FILTER_SANITIZE_STRING);
      $val = str_replace(',', '', $val);
      // compare if both id_feed are equal
      if ($val != null) {
        $arr['filter_type'] = $item->filter_type;
        $arr['value'] = $val;
        if($item->filter_type)
        $arr['alias_name'] = $id_feed_compare[0]->default_alias;
        // push arr values to 'field_value' array
        array_push($where_params, $arr);
      }
    }
  }

  // Setting 'date-desc' as the default sorted option
  $extra_params = [];
  $arr = [];
  $arr['sorted'] = filter_var($_GET['wovax-idx-select-sort'], FILTER_SANITIZE_STRING);
  if ($arr['sorted'] == null) {
    $arr['sorted'] = 'date-desc';
  }
  // if 'pagination' is available we´ll send more parameters in 'page_option'
  if ($attr[0]->pagination == "yes") {
    $arr['limit'] = $per_page;
    $arr['offset'] = ($paged - 1) * $per_page;
  }
  if ($_GET['wovax-idx-view-option'] == 'map'){
    $arr['limit'] = $attr[0]->per_map;
    $arr['offset'] = 0;
  }

  // push arr values to 'page_option' array
  array_push($extra_params, $arr);

  // set values in GET to evaluate next page (listing.php 11)
  $_GET['data_value'] = $feed_rules;
  $_GET['extra_params'] = $extra_params;
  $_GET['where_params'] = $where_params;
  // gets all data from API (helpers.php 233)
  $data2 = wovax_idx_get_data_list_bulding($extra_params, $where_params, $useless_var, $view_option, $api_key, $feed_rules);
  // insert data into the variable will be returned to listing.php
  if (is_object($data2)) {
    $data['data'] = $data2->data;
    $data['total'] = $data2->count;
  }
  else {
    $data = $data2;
  }
  return $data;
}

function wovax_idx_get_list_buildings_by_rules($attr, $rules, $useless_var, $view_option, $api_key, $feed_rules) {
  // $rules     = filter_var_array( $rules, FILTER_SANITIZE_STRING );
  $useless_var = filter_var($useless_var, FILTER_SANITIZE_STRING);
  /* $feeds_array = these feeds were delimited on shortcode admin feed tab */
  $feeds_array = json_encode(wovax_idx_return_id_feeds_array($attr[0]->feeds));
  /* $pagination = this was delimited on shortcode admin general tab */
  $pagination = $attr[0]->pagination;
  $per_page = $attr[0]->per_page;
  if (!empty($pagination) && !empty($per_page)) {

    // Pagination ON
    $paged = (get_query_var('page')) ? get_query_var('page') : 1;
  }
  // ================================ get id_field and value for every id_filter
  // *All id_feed in table FEEDS
  $id_feeds = wovax_idx_get_feeds_ids_by_shortcode($attr[0]->id);
  $where_params = [];

  // Loop to get and compare every id_feed from array
  $arr = [];
  if ($rules != '') {
    // Loop to get every id_field and push to the array
    foreach($rules as $key => $item) {
      $rule = wovax_idx_get_id_field_by_id_rule($item->id_rule);
      $rule = explode(",", $rule[0]->id_field);
      // rule´s id_feed
      $id_feed_compare = wovax_idx_get_id_feed_by_id_field($rule[0]);
      // compare if both id_feed are equal
      if ($item->rule_value != null) {
        $arr['filter_type'] = $item->rule_type;
        $arr['value'] = $item->rule_value;
        $arr['alias_name'] = $id_feed_compare[0]->default_alias;
        // push arr values to 'field_value' array
        array_push($where_params, $arr);
      }
    }
  }

  // Setting 'date-desc' as the default sorted option
  $extra_params = [];
  $arr = [];
  $arr['sorted'] = filter_var($_GET['wovax-idx-select-sort'], FILTER_SANITIZE_STRING);
  if ($arr['sorted'] == null) {
    $arr['sorted'] = 'date-desc';
  }
  // if 'pagination' is available we´ll send more parameters in 'page_option'
  if ($attr[0]->pagination == "yes") {
    $arr['limit'] = $per_page;
    $arr['offset'] = ($paged - 1) * $per_page;
  }
  if ($_GET['wovax-idx-view-option'] == 'map'){
    $arr['limit'] = $attr[0]->per_map;
    $arr['offset'] = 0;
  }

  // push arr values to 'page_option' array
  array_push($extra_params, $arr);

  // set values in GET to evaluate next page (listing.php 11)
  $_GET['data_value'] = $feed_rules;
  $_GET['extra_params'] = $extra_params;
  $_GET['where_params'] = $where_params;
  // gets all data from API (helpers.php 233)
  $data2 = wovax_idx_get_data_list_bulding($extra_params, $where_params, $useless_var, $view_option, $api_key, $feed_rules);
  // insert data into the variable will be returned to listing.php
  if (is_object($data2)) {
    $data['data'] = $data2->data;
    $data['total'] = $data2->count;
  }
  else {
    $data = $data2;
  }
  return $data;
}

function wovax_idx_get_list_buildings_by_favorites($attr, $favorites = [], $useless_var, $view_option, $api_key) {

  $useless_var = filter_var($useless_var, FILTER_SANITIZE_STRING);

  $pagination = $attr[0]->pagination;
  $per_page = $attr[0]->per_page;
  if (!empty($pagination) && !empty($per_page)) {

    // Pagination ON
    $paged = (get_query_var('page')) ? get_query_var('page') : 1;
  }

  // Setting 'date-desc' as the default sorted option
  $extra_params = [];
  $arr = [];
  $arr['sorted'] = filter_var($_GET['wovax-idx-select-sort'], FILTER_SANITIZE_STRING);
  if ($arr['sorted'] == null) {
    $arr['sorted'] = 'date-desc';
  }
  // if 'pagination' is available we´ll send more parameters in 'page_option'
  if ($attr[0]->pagination == "yes") {
    $arr['limit'] = $per_page;
    $arr['offset'] = ($paged - 1) * $per_page;
  }
  if ($_GET['wovax-idx-view-option'] == 'map'){
    $arr['limit'] = $attr[0]->per_map;
    $arr['offset'] = 0;
  }
  array_push($extra_params, $arr);

  //Create data_to_search
  //Insert all feeds in feeds_array without duplicates
  $feeds_array = [];
  foreach($favorites as $key => $value) {
    if (!in_array($value['0'], $feeds_array)) {
      array_push($feeds_array, $value['0']);
    }
  }

  //Insert every mls_number in each feeds_array index
  $fav_array = [];
  foreach ($feeds_array as $arr_feed_key => $arr_feed_id) {
    foreach ($favorites as $arr_fav_key => $arr_fav_values) {
      if($arr_feed_id == $arr_fav_values[0]){
        $fav_array[$arr_feed_id][] = $arr_fav_values[1];
      }
    }
  }

  $arr = [];
  $data_to_search = [];
  foreach ($fav_array as $key => $value) {
    $arr['class_id'] = $key;
    $arr['mls_id'] = $value;
    array_push($data_to_search, $arr);
  }

  // set values in GET to evaluate next page (listing.php 11)
  $_GET['data_to_search'] = $data_to_search;
  $_GET['extra_params'] = $extra_params;

  // gets all data from API (helpers.php 233)
  $data2 = wovax_idx_get_data_list_bulding_favorites($data_to_search, $extra_params, $useless_var, $view_option, $api_key);
  // insert data into the variable will be returned to listing.php
  if (is_object($data2)) {
    $data['data'] = $data2->data;
    $data['total'] = $data2->count;
  }
  else {
    $data = $data2;
  }
  return $data;
}

function wovax_idx_return_id_feeds_array($feeds) {
  $feeds = (empty($feeds)) ? array() : (json_decode($feeds));
  $feeds_array = [];
  foreach($feeds as $key => $value) {
    $indices_feeds = substr($value, 0, 2);
    $feeds_array[] = $indices_feeds;
  }
  return $feeds_array;
}

function wovax_idx_get_url() {
  global $post;
  $query_string = $_SERVER['QUERY_STRING'];
  return get_permalink($post->ID) . '?' . $query_string;
}

function wovax_idx_get_rules_by_shortcode($fields_array, $id_shortcode, $shortcode_type) {
  global $wpdb;
  $id_shortcode = filter_var($id_shortcode, FILTER_SANITIZE_NUMBER_INT);
  if (filter_var($id_shortcode, FILTER_VALIDATE_INT)) {
    $sql = "SELECT `feeds` FROM `{$wpdb->base_prefix}wovax_idx_shortcode` WHERE `id` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id_shortcode));
    $feeds = json_decode($results[0]->feeds);
    $feeds_string = "";
    $feed_value = array();
    if ($feeds != null) {
      foreach($feeds as $key => $value) {
        $feeds_ids = explode("-", $value);
        if (!in_array($feeds_ids[0], $feed_value)) {
          array_push($feed_value, $feeds_ids[0]);
          $feeds_string.= $feeds_ids[0] . ",";
        }
      }
      $feeds_string = rtrim($feeds_string, ',');
    }
    if ($feeds_string) {
      $feeds_string = explode(',', $feeds_string);
      $sql = "SELECT `id_feed` AS `class_id`, `id_field` AS `alias_name`, `field`, `rule_type` AS `type`, `rule_value` AS `rule` FROM `{$wpdb->base_prefix}wovax_idx_feed_rules` WHERE `id_feed` IN (";
      foreach ($feeds_string as $key => $value) {
        $sql .= "%d,";
      }
      $sql = substr($sql, 0, -1);
      $sql .= ") ORDER BY `alias_name`";
      $results = $wpdb->get_results($wpdb->prepare($sql, $feeds_string));
      $body = array();
      $rules_array = array();

      if ($shortcode_type == 'search_form'){

        foreach ($results as $key => $value) {
          if (!isset($rules_array[$value->alias_name])){
            $rules_array[$value->alias_name] = array();
          }
          array_push($rules_array[$value->alias_name], array( "type"  => $value->type,
                                                              "value" => $value->rule));
        }

        $body = $rules_array;

      }else{

        foreach ($feeds_string as $index => $feed) {
          $rule_class_body = array();
          $rule_class_body["class_id"] = $feed;
          $rule_class_body["rules"] = array();
          foreach ($results as $key => $value) {
              if ($value->class_id == $feed){
                array_push($rule_class_body["rules"], array(
                  "alias_name" => $value->field,
                  "type" => $value->type,
                  "rule" => $value->rule
                ));
              }
          }
          array_push($body, $rule_class_body);
        }

      }
      return $body;
    }else{
      return false;
    }
  }
  else {
    return false;
  }
}

function wovax_idx_get_feeds_by_shortcode($id_shortcode) {
  global $wpdb;
  $id_shortcode = filter_var($id_shortcode, FILTER_SANITIZE_NUMBER_INT);
  if (filter_var($id_shortcode, FILTER_VALIDATE_INT)) {
    $sql = "SELECT `feeds` FROM `{$wpdb->base_prefix}wovax_idx_shortcode` WHERE `id` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id_shortcode));
    $feeds = json_decode($results[0]->feeds);
    $feeds_string = "";
    $feed_value = array();
    if ($feeds != null) {
      foreach($feeds as $key => $value) {
        $feeds_ids = explode("-", $value);
        if (!in_array($feeds_ids[1], $feed_value)) {
          array_push($feed_value, $feeds_ids[1]);
          $feeds_string.= $feeds_ids[1] . ",";
        }
      }
      $feeds_string = rtrim($feeds_string, ',');
    }
    return $feeds_string;
  }
  else {
    return false;
  }
}

function wovax_idx_get_feeds_ids_by_shortcode($id_shortcode) {
  global $wpdb;
  $id_shortcode = filter_var($id_shortcode, FILTER_SANITIZE_NUMBER_INT);
  if (filter_var($id_shortcode, FILTER_VALIDATE_INT)) {
    $sql = "SELECT `feeds` FROM `{$wpdb->base_prefix}wovax_idx_shortcode` WHERE `id` = %d;";
    $results = $wpdb->get_results($wpdb->prepare($sql, $id_shortcode));
    $feeds = json_decode($results[0]->feeds);
    $feeds_string = "";
    $feed_value = array();
    if ($feeds != null) {
      foreach($feeds as $key => $value) {
        $feeds_ids = explode("-", $value);
        $feeds_string.= $feeds_ids[0] . ',';
      }
      $feeds_string = rtrim($feeds_string, ',');
    }
    return explode(",", $feeds_string);
  }
  else {
    return false;
  }
}

function wovax_idx_get_data_list_bulding_favorites($data_to_search, $extra_params, $useless_var, $view_option, $api_key) {
  $url       = "https://connect.wovax.com/api/my_mls_favorites";
  $extra_params[0]['grid_view'] = ($view_option == 'grid')?true:false;

  if (!empty($useless_var)) {
    $extra_params[0]['sorted'] = $useless_var;
  }
  $extra_params = str_replace(array(
    '[',
    ']'
  ) , '', htmlspecialchars(json_encode($extra_params) , ENT_NOQUOTES));
  $body = array(
    "data_to_search" => json_encode($data_to_search),
    "extra_params" => $extra_params
  );
  $rest = wovax_idx_remote_API_list_bulding($url, "", "POST", $body, $api_key);
  return $rest;
}

// ==============================================

// data for list building - listing view
function wovax_idx_get_data_list_bulding($extra_params, $where_params, $useless_var, $view_option, $api_key, $feed_rules) {
$url = ($view_option == 'grid')?"https://connect.wovax.com/api/search_resources":"https://connect.wovax.com/api/resources_geolocation";
  if (!empty($useless_var)) {
    $extra_params[0]['sorted'] = $useless_var;
  }
  $extra_params = str_replace(array(
    '[',
    ']'
  ) , '', htmlspecialchars(json_encode($extra_params) , ENT_NOQUOTES));
  $body = array(
    "data_values" => json_encode($feed_rules),
    "extra_params" => $extra_params,
    "where_params" => json_encode($where_params),
  );
  $rest = wovax_idx_remote_API_list_bulding($url, "", "POST", $body, $api_key);
  return $rest;
}

function wovax_idx_remote_API_list_bulding($url, $action, $type = "GET", $body = null, $api_key) {
  $response = wp_remote_post($url, array(
    'method' => $type,
    'timeout' => 45,
    'redirection' => 5,
    'httpversion' => '1.0',
    'blocking' => true,
    'headers' => array(
      "Authorization" => $api_key->data
    ),
    'body' => $body,
    'cookies' => array()
  ));
  if (is_wp_error($response)) {
    $error_message = $response->get_error_message();
    $wovax_data = 'error';
    return $wovax_data;
  }
  else {
    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body);
    if ($data->data == 'No data found') {
      return 'NO DATA FOUND - \'Empty data response\'';
    }
    elseif ($data->data == 'Unauthorised') {
      return 'UNAUTHORISED - \'API key denied\'';
    }
    elseif ($data->data == 'There is a problem with your setup') {
      return 'THERE IS A PROBLEM WITH YOUR SETUP - \'API in maintance\'';
    }
    elseif ($data->data == 'Data values is required') {
      return 'BAD REQUEST - \'There is a problem with your body request\'';
    }
    elseif ($data->data == null) {
      return 'PROBLEM IN RESPONSE - \'API isn´t sending data\'';
    }
    else {
      return $data;
    }
  }
}

function wovax_idx_remote_API_resource_details($url, $action, $type = "GET", $body = null) {
  $api_key = wovax_idx_get_validation_token();
  $response = wp_remote_post($url, array(
    'method' => $type,
    'timeout' => 45,
    'redirection' => 5,
    'httpversion' => '1.0',
    'blocking' => true,
    'headers' => array(
      "Authorization" => $api_key->data
    ),
    'body' => $body,
    'cookies' => array()
  ));
  if (is_wp_error($response)) {
    $error_message = $response->get_error_message();
    $wovax_data = 'error';
    return $wovax_data;
  }
  else {
    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body);
    if ($data->data == 'No data found') {
      return 'NO DATA FOUND - \'Empty data response\'';
    }
    elseif ($data->data == 'Unauthorised') {
      return 'UNAUTHORISED - \'API key denied by Wovax Connect\'';
    }
    elseif ($data->data == 'There is a problem with your setup') {
      return 'THERE IS A PROBLEM WITH YOUR SETUP - \'API in maintenance\'';
    }
    elseif ($data->data == 'Data values is required') {
      return 'BAD REQUEST - \'There is a problem with your body request\'';
    }
    elseif ($data->data == null) {
      return 'PROBLEM IN RESPONSE - \'The Wovax Connect API isn´t sending data\'';
    }
    else {
      $wovax_data = [];
      if (isset($data->data)) {
        return $data->data;
      }
      return $wovax_data;
    }
  }
}

function wovax_idx_get_data_filter($filters, $feeds, $api_key, $rules) {
  $array_general = array();
  $permission = (get_option("wovax-idx-settings-environment")=="production")?"production":"development";
  foreach($filters as $value) {
    if($value->filter_type == 'select'){

      $array_filter['filter_type'] = $value->filter_type;

      $alias_id_array_aux = explode(',', $value->id_field);
      $array_filter['alias_id'] = array(
        $alias_id_array_aux
      );

      $array_filter['alias_name'] = $value->field;

      $array_filter['rules'] = array();
      $rules_auxiliar = array();
      foreach ($rules as $rule => $val) {
        $val_key = $rule;
        if( in_array($val_key, $alias_id_array_aux)){
          $rules_auxiliar[$val_key] = $val;
        }
      }
      $array_filter['rules'] = $rules_auxiliar;
      array_push($array_general, $array_filter);
    }
  }
  $json_alias = json_encode($array_general);
  $url = "https://connect.wovax.com/api/alias_data";
  $body = array(
    "alias_data" => $json_alias,
    "feed_id" => "[" . $feeds . "]",
    "type_permission" => $permission
  );
  $rest = wovax_idx_remote_API_fields($url, "", "POST", $body, $api_key);
  return $rest;
}

// data for filters - search form view
function wovax_idx_remote_API_fields($url, $action, $type = "GET", $body = null, $api_key) {
  $response = wp_remote_post($url, array(
    'method' => $type,
    'timeout' => 45,
    'redirection' => 5,
    'httpversion' => '1.0',
    'blocking' => true,
    'headers' => array(
      "Authorization" => $api_key->data
    ),
    'body' => $body,
    'cookies' => array()
  ));
  if (is_wp_error($response)) {
    $error_message = $response->get_error_message();
    $wovax_data = 'error';
    return $wovax_data;
  }
  else {
    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body);
    $wovax_data = [];
    if ($data->data == 'No data found') {
      $wovax_data = 'PROBLEM IN RESPONSE - \'API isn´t sending data OR no filter has been selected\'';
    }
    elseif ($data->data == 'Unauthorised') {
      $wovax_data = 'UNAUTHORISED - \'API key denied\'';
    }
    elseif ($data->data == 'There is a problem with your setup') {
      $wovax_data = 'THERE IS A PROBLEM WITH YOUR SETUP - \'API in maintance\'';
    }
    elseif ($data->data == 'Data values is required') {
      $wovax_data = 'BAD REQUEST - \'There is a problem with your body request\'';
    }
    elseif ($data->data == null) {
      $wovax_data = 'PROBLEM IN RESPONSE - \'API isn´t sending data OR no filter has been selected\'';
    }
    else {
      if (isset($data->data)) {
        foreach($data as $key => $value) {
          array_push($wovax_data, $value);
        }
      }
    }
    return $wovax_data;
  }
}

function wovax_idx_get_validation_token() {
  $url = "https://connect.wovax.com/api/validate_domain";
  $body = array(
    "guest_email" => get_option('wovax-idx-settings-webmaster-email') ,
    "environment" => (get_option('wovax-idx-settings-environment') == 'production')?"production":"development"
  );
  
  $rest = wovax_idx_remote_API_validation_token($url, "", "POST", $body);
  return $rest;
}

function wovax_idx_remote_API_validation_token($url, $action, $type = "GET", $body = null) {
  $response = wp_remote_post($url, array(
    'method' => "POST",
    'timeout' => 45,
    'redirection' => 5,
    'httpversion' => '1.0',
    'blocking' => true,
    'headers' => array() ,
    'body' => $body,
    'cookies' => array()
  ));
  if (is_wp_error($response)) {
    return 'error';
  }
  else {
    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body);
    return $data;
  }
}
