<?php
namespace Wovax\IDX\Shortcodes;
// Exit if accessed directly
if (!defined('ABSPATH')) {
	exit();
}

abstract class Shortcode {
    private $errors  = array();
    private $scripts = array();
    private $styles  = array();
    private $tag     = '';
    private $loaded  = FALSE;
    public function __construct($tag) {
        $url = plugins_url('assets/css/shortcode.css', __FILE__);
        $this->addStyle('wovax-idx-shortcode', $url);
        $this->tag = 'wovax-idx-'.$tag;
        add_shortcode($this->tag, array($this, 'render'));
    }
    // Same as Wordpress's wp_enqueue_script for parameters.
    public function addScript($handle, $src = '', $deps = array(), $ver = FALSE, $in_footer = FALSE) {
        $this->scripts[$handle] = array($src, $deps, $ver, $in_footer);
    }
    // Same as Wordpress's wp_enqueue_style for parameters.
    public function addStyle($handle, $src = '', $deps = array(), $ver = FALSE, $media = 'all') {
        $this->styles[$handle] = array($src, $deps, $ver, $media);
    }
    public function enqueueScripts() {

    }
    public function enqueueStyles() {

    }
    public function render() {
        // Not a big fan of this but shortcodes in wordpress make pretty hard
        // to load CSS and JS in the header. Main reason being CSS not in the
        // head while valid in HTML 5 may block or cause delayed rendering.
        // However, the browsers handle it well but the stanard lets browers
        // choose the exact details. A shortcode can be called in lots of
        // places aka do_shortcode ect... so this is the best we can hope for.
        $this->loadResources();
        $html = $this->getContent();
        $html = $this->getErrorsHTML().$html;
        return $html;
    }
    public function tag(){
        return $this->tag;
    }
    // Error stuff
    public function addError($msg) {

        $this->errors[] = $msg;
    }
    public function getErrors() {
        return $this->errors;
    }
    private function getErrorsHTML() {
        $html = '';
        foreach($this->getErrors() as $msg) {
            $html .= '<div class="wovax-idx-shrt-warn"><p>'.esc_html($msg)."</p></div>\n";
        }
        return $html;
    }
    protected function loadResources() {
        if($this->loaded) {
            return;
        }
        foreach($this->scripts as $handle => $params) {
            wp_enqueue_script($handle, ...$params);
        }
        foreach($this->styles as $handle => $params) {
            wp_enqueue_style($handle, ...$params);
        }
        $this->loaded = TRUE;
    } 
    abstract protected function getContent();
}
  