<?php
// Exit if accessed directly
if (!defined('ABSPATH')) {
	exit();
}

use  Wovax\IDX\Utilities\ListingLoader;

function wovax_idx_get_content_listings_grid($results, $restriccions, $shortcode_type, $api_key) {
  // Global variable for user info
  global $current_user;
  $view_option = 'grid';
  $fav_avail = get_option('wovax-idx-settings-users-favorites');

  if($shortcode_type!='favorites'){
    $fields_array = array();
    foreach ($restriccions as $key => $value) {
      array_push($fields_array, $value->field);
    }

    $id = $results[0]->id;
    $feed_rules = wovax_idx_get_rules_by_shortcode($fields_array, $id, 'listing_embed');
  }

  wp_register_script('wovax-idx-feed-js', WOVAX_PLUGIN_NAME . 'admin/assets/js/wovax-idx-feed-general.js', array(
    'jquery'
  ) , '1.0', true);

  // Set variable in order to avoid error on calling
  $user_fav_properties = array();

  // Get user information to verify email, login, and ID
  if (is_user_logged_in()) {
    get_currentuserinfo();
    $user_id = $current_user->data->ID;

    // Sets favorites on user_meta in order to save favorite properties
    if (!isset(get_user_meta($user_id) ['wovax-idx-favorites'][0])) {
      update_user_meta($user_id, 'wovax-idx-favorites', null);
    }
    $user_fav_properties = (array)json_decode(get_user_meta($user_id) ['wovax-idx-favorites'][0]);
  }

  // Get sort option for calling data
  $sort_option = filter_var($_GET['wovax-idx-select-sort'], FILTER_SANITIZE_STRING);

  $pagination_auxiliar = $results[0]->pagination;
  $results[0]->pagination = 'yes';
  $results_per_page = ($results[0]->per_page)? $results[0]->per_page : 12;

  // Get data depending on shortcode type
  if ($shortcode_type == 'rules') {
    $data = (!empty($sort_option))?wovax_idx_get_list_buildings_by_rules($results, $restriccions, $sort_option, $view_option, $api_key, $feed_rules) : wovax_idx_get_list_buildings_by_rules($results, $restriccions, "", $view_option, $api_key, $feed_rules);
  } elseif ($shortcode_type == 'favorites') {
    $data = (!empty($sort_option))?wovax_idx_get_list_buildings_by_favorites($results, $user_fav_properties, $sort_option, $view_option, $api_key) : wovax_idx_get_list_buildings_by_favorites($results, $user_fav_properties, "", $view_option, $api_key);
  } else {
    $data = (!empty($sort_option))?wovax_idx_get_list_buildings_by_filters($results, $restriccions, $sort_option, $view_option, $api_key, $feed_rules) : wovax_idx_get_list_buildings_by_filters($results, $restriccions, "", $view_option, $api_key, $feed_rules);
  }

  if (is_array($data) && $data['total']) {

    // =================================================
    if($pagination_auxiliar != 'yes' && $data['total'] > $results_per_page) {
      $total_prop = $results_per_page;
    } else {
      $total_prop = $data['total'];
    }
    if( !empty( $results_per_page ) ) {
      $max_paged = ceil($total_prop/$results_per_page);
     } else {
      $max_paged = 1;
     }
    $data = $data['data'];
    $pagination_html = "";

    // =================================================
    // If user is not logged in, then he will in the modal.
    if (!is_user_logged_in()) {
      if (isset($_POST['email']) && isset($_POST['pass'])) {
        wovax_idx_custom_login();
      }
    }

    if ( ($results[0]->pagination == 'yes' && !empty($results_per_page)) ) {
      /* pagination */
      $actual_link = wovax_idx_get_url();
      $paged = (get_query_var('page')) ? get_query_var('page') : 1;
      $paged_next = $paged + 1;
      $paged_prev = $paged - 1;

      // Change page_option values to verify next page;
      $page_option = filter_var_array($_GET['extra_params'], FILTER_SANITIZE_STRING);
      $page_option[0]['offset'] = ($paged_next - 1) * filter_var($_GET['extra_params'][0]['limit'], FILTER_SANITIZE_STRING);
      $pagination_html = ($paged < $max_paged || $paged > 1) && ($pagination_auxiliar == 'yes') ? '<div style="float: left; width: 100%;text-align: center;" class="wx-nav-links">' : '';

      // evaluates if there exist more pages to be call in order to allow 'next' option. Favorite shortcode has a distinct function to evaluate it.
      /*
      if ($shortcode_type == 'favorites') {
        $verify_next_page = wovax_idx_get_data_list_bulding_favorites(filter_var_array($_GET['data_to_search'], FILTER_SANITIZE_STRING) , $page_option, "", $view_option, $api_key, $feed_rules);
      }else {
        $verify_next_page = wovax_idx_get_data_list_bulding($page_option, filter_var_array($_GET['where_params'], FILTER_SANITIZE_STRING) , "", $view_option, $api_key, $feed_rules);
      }
      */
      // if page is over 1 'prev' option is available
      if ($paged > 1 && $pagination_auxiliar == 'yes') {
        $pagination_html.= '<a href="' . esc_url($actual_link . '&page=' . $paged_prev) . '" id="wovax-idx-page-previous" >&lt; Previous |</a>';
      }

      if ( $paged_next <= $max_paged && $pagination_auxiliar == 'yes') {
        $pagination_html.= '<a href="' . esc_url($actual_link . '&page=' . $paged_next) . '" id="wovax-idx-page-next" >| Next &gt;</a>';
      }

      $pagination_html.= ($pagination_auxiliar == 'yes')?'</div>':'';
    }
    $select_sort_array = array(
      'date-desc' => 'Most Recent First',
      'date-asc' => 'Most Recent Last',
      'price-desc' => 'Price High to Low',
      'price-asc' => 'Price Low to High'
    );
    $output = '<div class="wovax-idx-action-bar-container col-6">
					        <form class="wovax-idx-form" action="" method="get" id="wovax-idx-sort">
							      <div class="wovax-idx-section">
								      <label style="line-height:68px;padding-left:16px;">' . esc_html(number_format($total_prop, 0, '', ',')) . ' ' . esc_html(($shortcode_type == 'favorites')?'Favorites':'Results') . '</label>
							      </div>
							      <div class="wovax-idx-section">';
    // Depending on shortcode type add input hidden for saving map value
    $output .= ($shortcode_type == 'rules' || $shortcode_type == 'favorites')?'<input type="hidden" id="wovax-idx-button-view-option" name="wovax-idx-view-option" value="' . esc_attr((!empty(filter_var($_GET['wovax-idx-view-option'], FILTER_SANITIZE_STRING)))?filter_var($_GET['wovax-idx-view-option'], FILTER_SANITIZE_STRING):$view_option) . '" >':'';

		$output .= '</div>
							  <div class="wovax-idx-section">
							  </div>
							  <div class="wovax-idx-section">
							  </div>';

    if ($results[0]->map_view != 'yes') {
			$output .= '<div class="wovax-idx-section">
							    </div>';
    }

		$output .= '<div class="wovax-idx-section">
								  <label for="sort">Sorted</label>
								  <select id="sort" name="wovax-idx-select-sort" onchange="(document.getElementById(\'form-tab-1\')== null)? this.form.submit() : document.getElementById(\'wovax-idx-select-sort-hidden\').value=this.value;document.getElementById(\'form-tab-1\').submit(); ">
								    <option value="" disabled>Select Sort Option</option>';

    // Setting all sorted option in select
    foreach($select_sort_array as $key => $value) {
      $selected = ($sort_option == null)? ($key == '')?'selected':'' : ($sort_option == $key)?'selected':'';
      $output .= '<option value="' . esc_attr($key) . '" ' . esc_attr($selected) . '>' . esc_html($value) . '</option>';
    }
    $output .= '</select></div>';

    // Depending on shortcode type change method for calling map view
    if ($results[0]->map_view == 'yes') {
      if ($shortcode_type == 'rules' || $shortcode_type == 'favorites') {
        $output .= '<div class="wovax-idx-section">
                      <a href="#" class="wovax-idx-button" onclick="wovax_idx_get_grid_view_listing(event);">Map View</a>
                    </div>';
      }else {
        $output .= '<div class="wovax-idx-section">
                      <a href="#" class="wovax-idx-button" onclick="wovax_idx_get_grid_view_search(event);">Map View</a>
                    </div>';
      }
    }
    $output .= '</form>
              </div>
              <input type="hidden" name="session_stat" value="' . esc_attr(is_user_logged_in()) . '">
              <div class="wovax-idx-listings-container col-3">';

    foreach($data as $key => $item) {
      // ===================================================================
      // getting format and currency for each property depending on its class_id
      $form_curren_id = json_decode(wovax_idx_get_feed_attr_by_id_feed($item->class_id)[0]->attributes);
      $format = $form_curren_id->format;
      $currency = $form_curren_id->currency;

      // ===================================================================
      // changing price format and currency
      $price = ($format === 'entire'?number_format($item->Price, 0, '', '') : ($format === 'miles'?number_format($item->Price, 0, '', ',') : ($format === 'decimals'?number_format($item->Price, 2, '.', '') : number_format($item->Price, 2, '.', ',') )));

      $price = $currency === 'right'?$price . "$":"$" . $price;

      // ===================================================================
      $main_photo = (empty($item->location)) ? 'https://cache.wovax.com/images/inspiration-de-42175.jpg' : $item->location;
      $desc_photo = (empty($item->photo_description)) ? '' : 'alt="' . esc_attr($item->photo_description) . '"';
      $street = "Street Address";
      $zip_code = "Zip Code";
      $sq_ft = "Square Footage";
      $ac_lt = "Acres";
      $mls = "MLS Number";
      $output .= '<div class="wovax-idx-section">
					          <div class="wovax-idx-listing-container">
                      <div class="wovax-idx-listing">
                        <a href="'.ListingLoader::buildURL($item->class_id, $item->id, $item).'">
                          <div class="wovax-idx-listing-image">
					            		  <img class="wovax-idx-listing-featured-image" src="' . esc_url('https://cache.wovax.com/image/?src=' . $main_photo . '&width=480&height=300&crop-to-fit') . '" ' . $desc_photo . '>';

      // Shows label "NEW" if it has been uploaded at least in the last 40 days
      $output .= '<div class="wovax-idx-listing-status">' . esc_html($item->Status) . '</div>';

      // Property details
      $output .= '</div>
					          <div class="wovax-idx-listing-content">
					            <div class="wovax-idx-listing-entry-meta">
					              <div class="wovax-idx-listing-title">' . esc_html($item->$street) . '<div class="wovax-idx-listing-subtitle">' . esc_html($item->City) . ' ' . esc_html($item->State) . ' ' . esc_html($item->$zip_code) . '</div>
  					            </div>';

      if($fav_avail == 'yes'){
    		$output .= '<div class="wovax-idx-listing-favorite" id="' . esc_attr($item->id) . '">';

        // Verify if the property is in User´s list to set already checked
        if (in_array(array($item->class_id, $item->$mls), $user_fav_properties) && is_user_logged_in()) {
          $output .= '<input class="toggle-heart" id="' . esc_attr($item->id) . '_heart" type="checkbox" checked onclick="wovax_idx_save_favorite_click(this)">';
        }
        else {
          $output .= '<input class="toggle-heart" id="' . esc_attr($item->id) . '_heart" type="checkbox" onclick="wovax_idx_save_favorite_click(this)">';
        }

        // If session is not started the modal href will be setted
        $output .= (!is_user_logged_in()) ? '<a href="#wovax-idx-login-registration-modal" onclick="wovax_idx_open_modal(this)">' : '';

        // Verify if the property is in User´s list to give style '&#xf101;'
        if (in_array(array($item->class_id, $item->$mls), $user_fav_properties) && is_user_logged_in()) {
          $output .= '<label id="toggle-heart-section" for="' . esc_attr($item->id) . '_heart" onclick="wovax_idx_change_heart(this)">&#xf101;</label>';
        }
        else {
          $output .= '<label id="toggle-heart-section" for="' . esc_attr($item->id) . '_heart" onclick="wovax_idx_change_heart(this)">&#xf100;</label>';
        }

        $output .= (!is_user_logged_in()) ? '</a>' : '';
        $output .= '</div>';
      }

			$output .= ' </div>
                    <input type="hidden" class="' . esc_attr($item->id) . '_feed" name="feed-id" value="' . esc_attr($item->class_id) . '">
                    <input type="hidden" class="' . esc_attr($item->id) . '_mls" name="feed-id" value="' . esc_attr($item->$mls) . '">
					          <div class="wovax-idx-listing-entry-content">
					            <div class="wovax-idx-listing-details">' . esc_html($item->Bedrooms) . ' Bedrooms + ' . esc_html($item->Bathrooms) . ' Bathrooms<br /> ' . esc_html($item->$sq_ft) . ' sq ft + ' . esc_html($item->$ac_lt) . ' acre lot
					            </div>
					          </div>
					          <div class="wovax-idx-listing-entry-footer">
					            <div class="wovax-idx-listing-price">' . esc_html($price) . '
					            </div>
					            <div class="wovax-idx-listing-mls-logo"><img src="https://cache.wovax.com/image/?src=https://cache.wovax.com/images/mls-board-logo.jpg">
					            </div>
					          </div>
					        </div>
		    			  </div>
					    </div>
			      </div>';
    }

    $output.= $pagination_html . '</div>';
    $output.= '<script>
            function wovax_idx_get_grid_view_search(event){
              event.preventDefault();
              jQuery("input:hidden[name=wovax-idx-view-option]").val("map");
              jQuery("#form-tab-1").submit();
            }
            function wovax_idx_get_grid_view_listing(event){
              event.preventDefault();
              jQuery("input:hidden[name=wovax-idx-view-option]").val("map");
              jQuery("#wovax-idx-sort").submit();
            }
						function wovax_idx_go_listing_details(e, url){
						    if(jQuery(e.target).attr("class") == "toggle-heart" || jQuery(e.target).attr("id") == "toggle-heart-section"){
						        return;
						    }
						    location.href = url;
						}
					</script>';
    return $output;
  }
  else {
    if (is_array($data)) {
      $data = "PROBLEM IN RESPONSE - 'No data found'";
    }
    return '<br />
			  <div class=warning>
    			<p>' . esc_html($data) . '</p>
			  </div>
			  <style media=screen type=text/css>
			  	.warning {
			  		font-size: 13px;
			  		padding-left: 15px;
					background: #FDECC4;
					border-left: 9px solid #FFB913;
					color: #2c3e50;
					padding-top: 20px;
					padding-bottom: 1px;
                }
              </style>';
  }
}
