<?php
namespace Wovax\IDX\Utilities;
// Exit if accessed directly
if (!defined('ABSPATH')) {
	exit();
}
use Wovax\IDX\API\WovaxConnect;
use Wovax\IDX\Settings\FeedFields;
use Wovax\IDX\Settings\IntialSetup;
use Wovax\IDX\Settings\SearchAppearance;
use Wovax\IDX\Shortcodes\ListingDetails;

class ListingLoader {
    private $idx_api     = NULL;
    private $listing     = NULL;
    private $init_opts   = NULL;
    private $short_code  = NULL;
    private $detail_base = '';
    private $detail_url  = '';
    public function __construct() {
        $this->idx_api    = WovaxConnect::createFromOptions();
        $this->init_opts  = new IntialSetup();
        $this->short_code = new ListingDetails();
        add_filter('parse_query', array($this, 'load'));  
    }
    public function echoMetaDescription() {
        $des = $this->getDescriptionString();
        if(strlen($des) > 0) {
            echo '<meta name="description" content="'.$des.'">'."\n";
        }
    }
    // TODO shorten this.
    public function load($query) {
        $detail_page_id = $this->init_opts->detailPage();
        if($detail_page_id < 1 || $query->queried_object_id !== $detail_page_id) {
            return;
        }
        $default = array(
            'wovax-idx-class-id' => 0,
            'wovax-idx-list-id'  => '0',
            'wovax-idx-view'     => 'details'
        );
        foreach($default as $key => $def) {
            if(!isset($query->query[$key])) {
                $query->query[$key] = $def;
            }
        }
        $view       = ($query->query['wovax-idx-view'] == 'details') ? 'details' : 'all';
        $class_id   = intval($query->query['wovax-idx-class-id']);
        $listing_id = trim($query->query['wovax-idx-list-id']);
        $fields     = new FeedFields($class_id);
        $data       = $this->idx_api->getListingDetails(
            $class_id,
            $listing_id,
            $fields->getRequired(),
            $view
        );
        if(!is_array($data)) {
            switch($data) {
                case 'No data found':
                    wp_redirect(get_permalink($this->init_opts->searchPage()), 301);
                    exit();
                    break;
                case 'Missing parameters':
                    $this->short_code->addError('No listing can be queried.');
                    return;
                    break;
                default:
                    $this->short_code->addError('Failure to load listing resources. Try again later.');
                    return;
                    break;
            }
        }
        $this->listing = $data;
        // Maybe redirect
        if (isset($_SERVER['HTTP_HOST'])) {
            $requested_url  = is_ssl() ? 'https://' : 'http://';
            $requested_url .= $_SERVER['HTTP_HOST'];
            $requested_url .= $_SERVER['REQUEST_URI'];
            $generated_url  = self::buildURL($class_id, $listing_id, $data);
            $requested      = @parse_url($requested_url);
            $generated      = @parse_url($generated_url);
            if($requested['path'] !== $generated['path']) {
                wp_redirect($generated_url, 301);
                exit();
            }
            $this->detail_url = $generated_url;
        }
        $this->short_code->addListing($class_id, $data);
        if($detail_page_id > 0) {
            $this->detail_base = get_permalink($detail_page_id);
            add_action('wp_head',              array($this, 'echoMetaDescription'));
            add_filter('document_title_parts', array($this, 'titleHeaderFilter'));
            add_filter('the_posts',            array($this, 'populatePostData'));
            add_filter('page_link',            array($this, 'overRidePageLink'));
            if(is_object($query->queried_object)) {
                $query->queried_object->post_title = $this->getTitleString(FALSE);
            }
        }
    }
    public function overRidePageLink($link) {
        if($link != $this->detail_base || strlen($this->detail_url) < 1) {
            return $link;
        }
        return $this->detail_url;
    }
    public function populatePostData($posts) {
        if(count($posts) < 1 || $posts[0]->ID !== $this->init_opts->detailPage()) {
            return $posts;
        }
        $post               = $posts[0];
        $post->post_title   = $this->getTitleString();
        $post->post_excerpt = $this->getDescriptionString();
        // Updating the post cache ensures that any call of get_post/get_instance uses our data.
        wp_cache_replace($post->ID, $post, 'posts');
        //update post
        $posts[0]            = $post;
        return $posts;
    }
    public function titleHeaderFilter($parts) {
        $parts['title'] = $this->getTitleString(FALSE);
        return $parts;
    }
    private function getDescriptionString() {
        $opts   = new SearchAppearance();
        $fields = $opts->getGlobalDescriptionFormat();
        $values = array();
        foreach($fields as $field) {
            if(!array_key_exists($field, $this->listing)) {
                continue;
            }
            $val = trim($this->listing[$field]);
            if(strlen($val) > 0) {
                $values[] = $val;
            }
        }
        return implode(' ', $values);
    }
    private function getTitleString($line_break = TRUE) {
        $opts  = new SearchAppearance();
        $parts = array();
        $sec   = 0;
        foreach($opts->getGlobalTitleFormat() as $item) {
            if(array_key_exists($sec, $parts) === FALSE) {
                $parts[$sec] = array();
            }
            if($item['type'] === 'value') {
                $field = $item['value'];
                $value = 'No '.$field.' Data';
                if(isset($this->listing[$field])) {
                    $tmp = trim($this->listing[$field]);
                    if(strlen($tmp) > 0) {
                        $value = $tmp;
                    }
                }
                $parts[$sec][] = $value;
            } else {
                $value = ' ';
                switch($item['value']) {
                    case 'line-break':
                        if($line_break) {
                            $value = ' <br>';
                        }
                        break;
                    case 'forward-slash':
                        $value = '/';
                        break;
                    case 'comma':
                        $value = ', ';
                        break;
                    case 'dash':
                        $value = ' - ';
                }
                $sec++;
                $parts[$sec] = array($value);
                $sec++;
            }
        }
        foreach($parts as $index => $val) {
            $parts[$index] = implode(' ', $val);
        }
        return implode('', $parts);
    }
    // Static methods
    public static function buildURL($class_id, $listing_id, $listing, $post_name = NULL) {
        $page_id = (new IntialSetup())->detailPage();
        if($page_id < 1) {
            return '';
        }
        $listing = (array)$listing; // make sure if it's an obj it's array
        $opts   = new SearchAppearance();
        $fields = $opts->getFeedUrlFormat($class_id);
        $values = array();
        foreach($fields as $field) {
            $value = 'null';
            if(array_key_exists($field, $listing)) {
                $value = self::escapeStrURL($listing[$field]);
            }
            $values[] = $value;
        }
        $prt = implode('/', $values);
        if(strlen($prt) > 0) {
            $prt .= '/';
        }
        $url  = home_url('/');
        $url .= get_post($page_id)->post_name.'/';
        $url .= 'wovax-idx/'.esc_html($class_id).'/';
        $url .= esc_html($listing_id).'/';
        $url .= $prt;
        return $url;
    }
    private static function escapeStrURL($str) {
        $str = strtolower($str);
        $str = preg_replace ('/[\s\\_\/.,:&-+]+/', '-', $str);
        $str = preg_replace ('/[^a-z0-9-]/', '', $str);
        $str = trim($str, '-');
        if(strlen($str) < 1) {
            return 'null';
        }
        return $str;
    }
}

$loader = new ListingLoader();
// Add query vars for URL rewriting.
add_filter('query_vars', function($qvars) {
    $qvars1 = array(
        'wovax-idx-class-id',
        'wovax-idx-list-id',
        'wovax-idx-view',
        'wovax-idx-field-vals'
    );
    return array_merge($qvars1, $qvars);
});
// Add URL rewriting.
add_filter('rewrite_rules_array', function($wp_rules) {
    $vars1 = array(
        'pagename',
        'wovax-idx-class-id',
        'wovax-idx-list-id',
        'wovax-idx-field-vals'
    );
    foreach($vars1 as $index => $val) {
        $vars1[$index] = $val.'=$matches['.($index+1).']';
    }
    $vars1[] = 'wovax-idx-view=details';
    $vars2 = $vars1;
    unset($vars1[3]);
	$url_rules = array(
        '^([^\/]+)\/wovax-idx\/([0-9]+)\/([^\/]+)\/?$' => 'index.php?'.implode('&', $vars1),
        '^([^\/]+)\/wovax-idx\/([0-9]+)\/([^\/]+)\/((?!.+\/\/).*[^\/])\/?$' => 'index.php?'.implode('&', $vars2)
        
    );
	return array_merge($url_rules, $wp_rules);
});