<?php
// Exit if accessed directly
if (!defined('ABSPATH')) {
	exit();
}
use Wovax\IDX\API\WovaxConnect;
use Wovax\IDX\Settings\FeedFields;
use Wovax\IDX\Utilities\ListingLoader;

/* This file will contain all the AJAX functions */
add_action('wp_ajax_search_page_post_title', 'wovax_idx_search_page_post_title');
add_action('wp_ajax_create_page_post', 'wovax_idx_create_page_post');
add_action('wp_ajax_create_listing_page_post', 'wovax_idx_create_listing_page_post');
add_action('wp_ajax_get_wovax_idx_get_result_api', 'wovax_idx_get_result_api');
add_action('wp_ajax_get_wovax_idx_get_result_api_feed_details', 'wovax_idx_get_result_api_feed_details');
add_action('wp_ajax_get_wovax_idx_get_result_api_feed_for_search_content', 'wovax_idx_get_result_api_feed_for_search_content');
add_action('wp_ajax_set_order_section', 'wovax_idx_set_order_section');
add_action('wp_ajax_get_wovax_idx_sort_shortcode', 'wovax_idx_get_wovax_idx_sort_shortcode');
add_action('wp_ajax_get_wovax_idx_sort_user_activity', 'wovax_idx_get_wovax_idx_sort_user_activity');
add_action('wp_ajax_save_favorite', 'wovax_idx_save_favorite');
add_action('wp_ajax_delete_favorite', 'wovax_idx_delete_favorite');
add_action('wp_ajax_update_profile', 'wovax_idx_update_profile');
add_action('wp_ajax_get_infobox_marker', 'wovax_idx_get_infobox_marker');
add_action('wp_ajax_nopriv_get_infobox_marker', 'wovax_idx_get_infobox_marker');


/* function to sort the shortcode table */
function wovax_idx_get_wovax_idx_sort_user_activity() {
  if (!isset($_POST['attr'])) {
    echo json_encode(array());
    die();
  }

  $orderby = filter_var($_POST['attr']['orderby'], FILTER_SANITIZE_STRING);
  $ordername = filter_var($_POST['attr']['ordername'], FILTER_SANITIZE_STRING);
  $search = filter_var($_POST['attr']['search'], FILTER_SANITIZE_STRING);
  $paged = filter_var($_POST['attr']['paged'], FILTER_SANITIZE_STRING);

  $per_page = get_option('wovax-idx-users-per-page');
  $result = wovax_idx_user_table_sort($per_page, $paged, $orderby, $ordername, $search);
  $result = json_decode($result);
  if ($result->count_total && $result->data && !empty($result->data) && !empty($result->count_total)) {
  
    $total = $result->count_total;
    $pag_pages = ceil($total / $per_page);
    $table_html = '';
    $paginate_html = '';

    $user = get_current_user_id();
    $user_meta = get_user_meta( $user, 'managewovax-idx_page_wovax_idx_user_activitycolumnshidden', true );
    $user_meta = ( is_array($user_meta) )?$user_meta:array(); 

    $columns_count = 7 - ( count($user_meta) / 2 );
    
    $fullname_hidden  = in_array('fullname', $user_meta)?' hidden':'';
    $phone_hidden     = in_array('phone', $user_meta)?' hidden':'';
    $email_hidden     = in_array('email', $user_meta)?' hidden':'';
    $favorites_hidden = in_array('favorites', $user_meta)?' hidden':'';

    foreach ($result->data as $key => $item) {
      $item = ( object )$item;

      $table_html .= '<tr>
                           <td class="title column-title has-row-actions column-primary" data-colname="Class">
                            ' . $item->picture . '<strong><a class="row-title" href="' . esc_url('admin.php?page=wovax_idx_users&amp;tab=general&amp;action=update&amp;iduser=' . $item->id . '') . '" title="Edit ' . esc_attr($item->nickname) . '">' . esc_html($item->nickname) . '</a></strong>
                            <div class="row-actions"><span class="edit"><a href="' . esc_url('admin.php?page=wovax_idx_users&amp;tab=general&amp;action=update&amp;iduser=' . $item->id . '') . '">Details</a></span></div>
                            <button type="button" class="toggle-row"><span class="screen-reader-text">Show more details</span></button>
                           </td>
                           <td class="fullname column-fullname' . esc_attr($fullname_hidden) . '" data-colname="Fullname">
                             ' . esc_html($item->username) . '
                           </td>
                           <td class="phone column-phone' . esc_attr($phone_hidden) . '" data-colname="Phone">
                             ' . esc_html($item->phone) . '
                           </td>
                           <td class="email column-email' . esc_attr($email_hidden) . '" data-colname="Email">
                             ' . esc_html($item->email) . '
                           </td>
                           <td class="favorites column-favorites' . esc_attr($favorites_hidden) . '" data-colname="Favorites">
                             ' . esc_html(($item->qty_prop>1?$item->qty_prop . ' Properties': ($item->qty_prop==0?'None':$item->qty_prop . ' Property'))) . '
                           </td>
                         </tr>';
    }
  

    // PAGINATION
    if ($paged < $pag_pages && $paged == '1') {
      $wovax_idx_pag_prev = '<span class="tablenav-pages-navspan" aria-hidden="true">«</span>
                   <span class="tablenav-pages-navspan" aria-hidden="true">‹</span>';
      $wovax_idx_pag_next = '<a href="#" id="wovax-idx-users-next-search" class="next-page"><span class="screen-reader-text" >Next page</span><span aria-hidden="true">›</span></a>
                   <a class="last-page" href="#" id="wovax-idx-users-last-search"><span class="screen-reader-text">Last page</span><span aria-hidden="true">»</span></a>';
    }
    elseif ($paged < $pag_pages) {
      $wovax_idx_pag_prev = '<a class="first-page" href="#" id="wovax-idx-users-first-search"><span class="screen-reader-text">First page</span><span aria-hidden="true">«</span></a>
                  <a class="prev-page" href="#" id="wovax-idx-users-prev-search"><span class="screen-reader-text">Previous page</span><span aria-hidden="true">‹</span></a>';
      $wovax_idx_pag_next = '<a href="#" id="wovax-idx-users-next-search" class="next-page"><span class="screen-reader-text" >Next page</span><span aria-hidden="true">›</span></a>
                   <a class="last-page" href="#" id="wovax-idx-users-last-search"><span class="screen-reader-text">Last page</span><span aria-hidden="true">»</span></a>';
    }
    else {
      $wovax_idx_pag_prev = '<a class="first-page" href="#" id="wovax-idx-users-first-search"><span class="screen-reader-text">First page</span><span aria-hidden="true">«</span></a>
                    <a class="prev-page" href="#" id="wovax-idx-users-prev-search"><span class="screen-reader-text">Previous page</span><span aria-hidden="true">‹</span></a>';
      $wovax_idx_pag_next = '<span class="tablenav-pages-navspan" aria-hidden="true">›</span>
                  <span class="tablenav-pages-navspan" aria-hidden="true">»</span>';
    }

    if ($total <= $per_page) {
      $paginate_html.= '
                <div class="tablenav-pages">
                  <span class="displaying-num">' . esc_html($total) . ' items</span>
                </div>';
    }
    else {
      $paginate_html.= '
                <div class="tablenav-pages">
                  <span class="displaying-num">' . esc_html($total) . ' items</span>
                    <span class="pagination-links">
                      ' . $wovax_idx_pag_prev . '
                      <span class="paging-input">
                        <label for="wovax-idx-current-users-page-selector" class="screen-reader-text">Actual page</label>
                        <input class="current-page" id="wovax-idx-current-users-page-selector" type="text" name="paged" value="' . esc_attr($paged) . '" size="1" aria-describedby="table-paging">
                        <span class="tablenav-paging-text"> of <span class="total-pages" id="wovax-idx-users-pag-pages">' . esc_html($pag_pages) . '</span></span>
                      </span>
                        ' . $wovax_idx_pag_next . '
                    </span>
                </div>';
    }

    echo json_encode(array(
      'table_html' => $table_html,
      'paginate_html' => $paginate_html
    ));
    die();
  }
  else {
    $table_html.= '<tr><td class="colspanchange" colspan="' . esc_html($columns_count) . '">No users found.</td></tr>';
  }

  echo json_encode(array(
    'table_html' => $table_html
  ));
  die();
}

function wovax_idx_get_wovax_idx_sort_shortcode() {
  $orderby = filter_var($_POST['attr']['orderby'], FILTER_SANITIZE_STRING);
  $ordername = filter_var($_POST['attr']['ordername'], FILTER_SANITIZE_STRING);
  $search = filter_var($_POST['attr']['search'], FILTER_SANITIZE_STRING);
  $filter = filter_var($_POST['attr']['filter'], FILTER_SANITIZE_STRING);
  $section = filter_var($_POST['attr']['section'], FILTER_SANITIZE_STRING);
  $result = wovax_idx_shortcode_table_sort($orderby, $ordername, $search, $filter, $section);
  $shortcode_table = '';
  
  $user = get_current_user_id();
  $shortcode_meta = get_user_meta( $user, 'managewovax-idx_page_wovax_idx_shortcodescolumnshidden', true );
  $shortcode_meta = ( is_array($shortcode_meta) )?$shortcode_meta:array();

  $shortcode_hidden = in_array('shortcode', $shortcode_meta)?' hidden':'';
  $type_hidden      = in_array('type', $shortcode_meta)?' hidden':'';
  $author_hidden    = in_array('author', $shortcode_meta)?' hidden':'';
  $created_hidden   = in_array('created', $shortcode_meta)?' hidden':'';

  if ($section == 'trash') {
    foreach($result as $key => $item) {
      $count++;
      $date_mys = mysql2date('U', $item->date);
      $last_updated = esc_html(human_time_diff($date_mys, current_time('timestamp')));
      $filter_type = ($item->type == 'listings' ? 'Listings Embed' : ($item->type == 'search_form' ? 'Search Form' : ($item->type == 'user_favorites' ? 'User Favorites' : 'User Profile')));
      $shortcode_table.= '<tr>
				                    <th scope="row" class="check-column">
				                      <input type="checkbox" name="post[]" value="' . esc_attr($item->id) . '">
				                    </th>
				                    <td class="title column-title has-row-actions column-primary" data-colname="Title">
				                      <strong><a class="row-title" "' . esc_url('admin.php?page=wovax_idx_shortcodes&action=update&id=' . $item->id . '') . '" title="Edit “Main site search”">' . esc_html($item->title) . '</a></strong>
				                      <div class="row-actions"><span class="untrash"><a  href="' . esc_url('admin.php?page=wovax_idx_shortcodes&action=untrash&t=t&id=' . $item->id . '') . '" aria-label="Restore “dfg” from the Trash">Restore</a> | </span><span class="delete"><a href="' . esc_url('admin.php?page=wovax_idx_shortcodes&action=delete&t=t&id=' . $item->id . '') . '" class="submitdelete" aria-label="Delete “dfg” permanently">Delete Permanently</a></span></div>
				                      <button type="button" class="toggle-row"><span class="screen-reader-text">Show more details</span></button>
				                    </td>
				                    <td class="shortcode column-shortcode' . esc_attr($shortcode_hidden) . '" data-colname="Shortcode">
				                      [wovax-idx id="' . esc_html($item->id) . '"]
				                    </td>
				                    <td class="type column-type' . esc_attr($type_hidden) . '" data-colname="Type">
				                     ' . esc_html($filter_type) . '
				                    </td>
				                    <td class="author column-author' . esc_attr($author_hidden) . '" data-colname="Author">
				                       ' . esc_html($item->author) . '
				                    </td>
				                    <td class="created column-created' . esc_attr($created_hidden) . '" data-colname="Created">
				                      <abbr title="' . esc_attr($item->date) . '">
				                       ' . esc_html($last_updated) . '
				                      </abbr>
				                    </td>
				                  </tr>';
    }
  }
  else {
    foreach($result as $key => $item) {
      $count++;
      $date_mys = mysql2date('U', $item->date);
      $last_updated = esc_html(human_time_diff($date_mys, current_time('timestamp')));
      $filter_type = ($item->type == 'listings' ? 'Listings Embed' : ($item->type == 'search_form' ? 'Search Form' : ($item->type == 'user_favorites' ? 'User Favorites' : 'User Profile')));
      $shortcode_table.= '<tr>
			                    <th scope="row" class="check-column">
			                      <input type="checkbox" name="post[]" value="' . esc_attr($item->id) . '">
			                    </th>
			                    <td class="title column-title has-row-actions column-primary" data-colname="Title">
			                      <strong><a class="row-title" "' . esc_url('admin.php?page=wovax_idx_shortcodes&action=update&id=' . $item->id . '') . '" title="Edit “Main site search”">' . esc_html($item->title) . '</a></strong>
			                      <div class="row-actions"><span class="edit"><a href="' . esc_url('admin.php?page=wovax_idx_shortcodes&action=update&id=' . $item->id . '') . '">Edit</a> | </span><span class="copy"><a href="' . esc_url('admin.php?page=wovax_idx_shortcodes&id=' . $item->id . '&action=duplicate') . '">Duplicate</a> | </span><span class="trash"><a href="' . esc_url('admin.php?page=wovax_idx_shortcodes&id=' . $item->id . '&action=trash') . '" class="submitdelete" aria-label="Move “Main site search” to the Trash">Trash</a></span></div>
			                      <button type="button" class="toggle-row"><span class="screen-reader-text">Show more details</span></button>
			                    </td>
			                    <td class="shortcode column-shortcode' . esc_attr($shortcode_hidden) . '" data-colname="Shortcode">
			                      [wovax-idx id="' . esc_html($item->id) . '"]
			                    </td>
			                    <td class="type column-type' . esc_attr($type_hidden) . '" data-colname="Type">
			                     ' . esc_html($filter_type) . '
			                    </td>
			                    <td class="author column-author' . esc_attr($author_hidden) . '" data-colname="Author">
			                       ' . esc_html($item->author) . '
			                    </td>
			                    <td class="created column-created' . esc_attr($created_hidden) . '" data-colname="Created">
			                      <abbr title="' . esc_attr($item->date) . '">
			                       ' . esc_html($last_updated) . '
			                      </abbr>
			                    </td>
			                  </tr>';
    }
  }

  if (empty($result)) {
    $shortcode_table = '<tr><td class="colspanchange" colspan="6">No shortcodes found.</td></tr>';;
  }

  $wovax_table = array(
    'table' => $shortcode_table
  );
  echo json_encode($wovax_table);
  die();
}

/* function to retrieve the page or post list */
function wovax_idx_search_page_post_title() {
  $wovax_idx_name = filter_var($_POST['wovax_idx_name'], FILTER_SANITIZE_STRING);
  $list_titles = wovax_idx_get_all_post_page($wovax_idx_name);
  echo json_encode(array(
    'page_post' => $list_titles
  ));
  die();
}

/* function to create a new page and retrieves post id */
function wovax_idx_create_page_post() {
  $title_post = "Search Results";
  $id_post = wovax_idx_create_post_page($title_post, '[wovax-idx-search-results]');
  echo json_encode(array(
    'id_post' => $id_post
  ));
  die();
}

/* function to create a new page for Listing Details and retrieves post id */
function wovax_idx_create_listing_page_post() {
  $title_post = "Listing Details";
  $id_post = wovax_idx_create_post_page($title_post, '[wovax-idx-listing-details]');
  echo json_encode(array(
    'id_post' => $id_post
  ));
  die();
}

/* function to retrieve all rows in the table and the pagination */
function wovax_idx_get_result_api() {
  if (!isset($_POST['attr'])) {
    echo json_encode(array());
    die();
  }

  $orderby = filter_var($_POST['attr']['orderby'], FILTER_SANITIZE_STRING);
  $ordername = filter_var($_POST['attr']['ordername'], FILTER_SANITIZE_STRING);
  $s = filter_var($_POST['attr']['s'], FILTER_SANITIZE_STRING);
  $paged = filter_var($_POST['attr']['paged'], FILTER_SANITIZE_STRING);
  $filter = filter_var($_POST['attr']['filter'], FILTER_SANITIZE_STRING);

  $per_page = get_option('wovax-idx-feeds-per-page');
  $result = wovax_idx_feed_request_data($per_page, $paged, $orderby, $ordername, $s, $filter);
  $result = json_decode($result);
  if ($result->count_total && $result->data && !empty($result->data) && !empty($result->count_total)) {
    
    $total = $result->count_total;
    $pag_pages = ceil($total / $per_page);
    $table_html = '';
    $paginate_html = '';

    $lista_option = array();
    $lista_complete = wovax_idx_merge_list_feeds($lista_option, $result->data);
    update_option('wovax_idx_feeds_list', json_encode($lista_complete));

    $user = get_current_user_id();
    $feed_meta = get_user_meta( $user, 'managewovax-idx_page_wovax_idx_feedscolumnshidden', true );
    $feed_meta = ( is_array($feed_meta) )?$feed_meta:array();

    $columns_count = 7 - ( count($feed_meta) / 2 );
    
    $resource_hidden    = in_array('resource', $feed_meta)?' hidden':'';
    $feed_hidden        = in_array('feed', $feed_meta)?' hidden':'';
    $board_hidden       = in_array('board', $feed_meta)?' hidden':'';
    $environment_hidden = in_array('environment', $feed_meta)?' hidden':'';
    $status_hidden      = in_array('status', $feed_meta)?' hidden':'';
    $updated_hidden     = in_array('updated', $feed_meta)?' hidden':'';
    // TABLE HTML
    foreach($result->data as $key => $item) {


      $item = ( object )$item;
      $status = ($item->status == '1') ? 'Active' : 'Inactive';
      $table_html.= '
				<tr>
					<td class="title column-title has-row-actions column-primary" data-colname="Class">
			          <strong><a class="row-title" href="' . esc_url('admin.php?page=wovax_idx_feeds&tab=general&action=update&idfeed=' . $item->class_id . '') . '" title="Edit Residential" >' . esc_html($item->class_visible_name) . '</a></strong>
			          <div class="row-actions"><span class="edit"><a href="' . esc_url('admin.php?page=wovax_idx_feeds&tab=general&action=update&idfeed=' . $item->class_id . '') . '" >Edit</a></span></div>
			          <button type="button" class="toggle-row"><span class="screen-reader-text">Show more details</span></button>
			        </td>
			        <td class="resource column-resource' . esc_attr($resource_hidden) . '" data-colname="Resource">
			          ' . esc_html($item->resource) . '
			        </td>
			        <td class="feed column-feed' . esc_attr($feed_hidden) . '" data-colname="Feed">
			         ' . esc_html($item->feed_description) . '
			        </td>
			        <td class="board column-board' . esc_attr($board_hidden) . '" data-colname="Board">
			          ' . esc_html($item->board_acronym) . '
			        </td>
			        <td class="environment column-environment' . esc_attr($environment_hidden) . '" data-colname="Environment">
			          ' . esc_html($item->environment) . '
			        </td>
			        <td class="status column-status' . esc_attr($status_hidden) . '" data-colname="Status">
			          ' . esc_html($status) . '
			        </td>
			        <td class="updated column-updated' . esc_attr($updated_hidden) . '" data-colname="Updated">
			          <abbr title="' . esc_attr($item->updated) . '">' . esc_html($item->updated) . '</abbr>
			        </td>
			      </tr>
			      ';
    }

    // PAGINATION
    if ($paged < $pag_pages && $paged == '1') {
      $wovax_idx_pag_prev = '<span class="tablenav-pages-navspan" aria-hidden="true">«</span>
								   <span class="tablenav-pages-navspan" aria-hidden="true">‹</span>';
      $wovax_idx_pag_next = '<a href="#" id="wovax-idx-feeds-next-search" class="next-page"><span class="screen-reader-text" >Next page</span><span aria-hidden="true">›</span></a>
								   <a class="last-page" href="#" id="wovax-idx-feeds-last-search"><span class="screen-reader-text">Last page</span><span aria-hidden="true">»</span></a>';
    }
    elseif ($paged < $pag_pages) {
      $wovax_idx_pag_prev = '<a class="first-page" href="#" id="wovax-idx-feeds-first-search"><span class="screen-reader-text">First page</span><span aria-hidden="true">«</span></a>
									<a class="prev-page" href="#" id="wovax-idx-feeds-prev-search"><span class="screen-reader-text">Previous page</span><span aria-hidden="true">‹</span></a>';
      $wovax_idx_pag_next = '<a href="#" id="wovax-idx-feeds-next-search" class="next-page"><span class="screen-reader-text" >Next page</span><span aria-hidden="true">›</span></a>
								   <a class="last-page" href="#" id="wovax-idx-feeds-last-search"><span class="screen-reader-text">Last page</span><span aria-hidden="true">»</span></a>';
    }
    else {
      $wovax_idx_pag_prev = '<a class="first-page" href="#" id="wovax-idx-feeds-first-search"><span class="screen-reader-text">First page</span><span aria-hidden="true">«</span></a>
								    <a class="prev-page" href="#" id="wovax-idx-feeds-prev-search"><span class="screen-reader-text">Previous page</span><span aria-hidden="true">‹</span></a>';
      $wovax_idx_pag_next = '<span class="tablenav-pages-navspan" aria-hidden="true">›</span>
									<span class="tablenav-pages-navspan" aria-hidden="true">»</span>';
    }

    if ($total <= $per_page) {
      $paginate_html.= '
								<div class="tablenav-pages">
									<span class="displaying-num">' . esc_html($total) . ' items</span>
								</div>';
    }
    else {
      $paginate_html.= '
								<div class="tablenav-pages">
									<span class="displaying-num">' . esc_html($total) . ' items</span>
										<span class="pagination-links">
											' . $wovax_idx_pag_prev . '
											<span class="paging-input">
												<label for="wovax-idx-current-feeds-page-selector" class="screen-reader-text">Actual page</label>
												<input class="current-page" id="wovax-idx-current-feeds-page-selector" type="text" name="paged" value="' . esc_attr($paged) . '" size="1" aria-describedby="table-paging">
												<span class="tablenav-paging-text"> of <span class="total-pages" id="wovax-idx-feeds-pag-pages">' . esc_html($pag_pages) . '</span></span>
											</span>
											 	' . $wovax_idx_pag_next . '
										</span>
								</div>';
    }

    echo json_encode(array(
      'table_html' => $table_html,
      'paginate_html' => $paginate_html
    ));
    die();
  }
  else {
    $table_html.= '<tr><td class="colspanchange" colspan="' . esc_html($columns_count) . '">No feeds found.</td></tr>';
  }

  echo json_encode(array(
    'table_html' => $table_html
  ));
  die();
}

function wovax_idx_get_infobox_marker() {
    $list_id  = filter_var( $_POST['id'], FILTER_SANITIZE_STRING);
    $class_id = filter_var( $_POST['class'], FILTER_SANITIZE_STRING);
    $user_fav_properties = filter_var_array( $_POST['prop'], FILTER_SANITIZE_STRING);
    $fav_avail = filter_var( $_POST['fav_avail'], FILTER_SANITIZE_STRING);

    $form_curren_id = json_decode(wovax_idx_get_feed_attr_by_id_feed($class_id)[0]->attributes);
    $format = $form_curren_id->format;
    $currency = $form_curren_id->currency;
    $idx_api    = WovaxConnect::createFromOptions();
    $fields     = new FeedFields($class_id);
    $rest       = $idx_api->getListingDetails(
        $class_id,
        $list_id,
        $fields->getRequired(),
        'single'
    );
    $rest = (object) $rest;
    $price = ($format === 'entire'?number_format($rest->Price, 0, '', '') : ($format === 'miles'?number_format($rest->Price, 0, '', ',') : ($format === 'decimals'?number_format($rest->Price, 2, '.', '') : number_format($rest->Price, 2, '.', ',') )));
    $price = $currency === 'right'?$price . "$" : "$" . $price;

    $photo_source = 'https://cache.wovax.com/images/inspiration-de-42175.jpg';
    $photo_alt    = '';
    if(count($rest->photos_list) > 0) {
        $photo_source = $rest->photos_list[0]['location'];
        if(strlen($rest->photos_list[0]['description']) > 0) {
            $photo_alt = $rest->photos_list[0]['description'];
        }
    }
    $zip_code = 'Zip Code';
    $sq_ft = "Square Footage";
    $ac_lt = "Lot Size";
    $street_address = 'Street Address';
    $mls = 'MLS Number';

  $output =  '<div class="wovax-idx-listing-container">
                    <div class="wovax-idx-listing">
                        <a href="'.ListingLoader::buildURL($class_id, $list_id, $rest).'">
                            <div class="wovax-idx-listing-image">
                                <img alt="'.esc_html($photo_alt).'" class="wovax-idx-listing-featured-image" src="https://cache.wovax.com/image/?src='.esc_url($photo_source).'&width=480&height=300&crop-to-fit">';

    // No idea what interval is. However, it's not defined so this always evaluates as FALSE.
    //$output.= ($interval > - 40) ? '<div class="wovax-idx-listing-status">New</div>' : '';

  $output.=  '</div>
                <div class="wovax-idx-listing-content">
                  <div class="wovax-idx-listing-entry-meta">
                    <div class="wovax-idx-listing-title">' . esc_html($rest->$street_address) . '<div class="wovax-idx-listing-subtitle">' . esc_html($rest->City) . ' ' . esc_html($rest->State) . ' ' . esc_html($rest->$zip_code) . '</div>
                  </div>';
  if($fav_avail == 'yes'){
    $output.=  ' <div class="wovax-idx-listing-favorite" id="' . esc_attr($list_id) . '">';

    if (in_array(array($class_id, $rest->$mls), $user_fav_properties) && is_user_logged_in()) {
      $output.= '<input class="toggle-heart" id="' . esc_attr($list_id) . '_heart" type="checkbox" checked onclick="wovax_idx_save_favorite_click(this)">';
    }
    else {
      $output.= '<input class="toggle-heart" id="' . esc_attr($list_id) . '_heart" type="checkbox" onclick="wovax_idx_save_favorite_click(this)">';
    }

    $output.= (!is_user_logged_in()) ? '<a href="#wovax-idx-login-registration-modal" rel="modal:open" onclick="wovax_idx_open_modal(this)">' : '';

    if (in_array(array($class_id, $rest->$mls), $user_fav_properties) && is_user_logged_in()) {
      $output.= '<label id="toggle-heart-section" for="' . esc_attr($list_id) . '_heart" onclick="wovax_idx_change_heart(this)">&#xf101;</label>';
    }
    else {
      $output.= '<label id="toggle-heart-section" for="' . esc_attr($list_id) . '_heart" onclick="wovax_idx_change_heart(this)">&#xf100;</label>';
    }

    $output.= (!is_user_logged_in()) ? '</a>' : '';
    $output.=  '</div>';
  }
  $output.=  ' </div>
                  <input type="hidden" class="' . esc_attr($list_id) . '_feed" name="feed-id" value="' . esc_attr($class_id) . '">
                  <input type="hidden" class="' . esc_attr($list_id) . '_mls" name="feed-id" value="' . esc_attr($rest->$mls) . '">
                  <div class="wovax-idx-listing-entry-content">
                    <div class="wovax-idx-listing-details">' . esc_html($rest->Bedrooms) . ' Bedrooms + ' . esc_html($rest->Bathrooms) . ' Bathrooms<br /> ' . esc_html($rest->$sq_ft) . ' sq ft + ' . esc_html($rest->$ac_lt) . ' acre lot
                    </div>
                  </div>
                  <div class="wovax-idx-listing-entry-footer">
                    <div class="wovax-idx-listing-price">' . esc_html($price) . '
                    </div>
                    <div class="wovax-idx-listing-mls-logo"><img src="https://cache.wovax.com/image/?src=https://cache.wovax.com/images/mls-board-logo.jpg">
                    </div>
                  </div>
                </div>
              </div>
            </div>
          ';
  echo $output;

  die();
}

/* function to retrieve all fields into the wovax_IDX_detail form */
function wovax_idx_get_result_api_feed_details() {
  if (!isset($_POST['attr'])) {
    echo json_encode(array());
    die();
  }

  $id_feed = filter_var($_POST['attr']['idfeed'], FILTER_SANITIZE_STRING);
  $api_data = wovax_idx_api_data_feed_details($id_feed);

  // $result  	 = json_decode( $api_data );
  $all = '';
  $default = '';
  $values_array = array('City', 'State', 'Zip Code', 'Bedrooms', 'Bathrooms', 'Price', 'Description');
  $list_result = wovax_idx_list_feeds_field_by_id($id_feed);

  if ($api_data[0][0] == 'No data found') {
    $all.= '';
    $default.= '';
  }
  else {
    $all.='<li>
              <label class="menu-item-title">
                <input type="checkbox" class="menu-item-checkbox" name="wovax_idx_map_option" value="Map"> Map
              </label>
              <input type="hidden" name="wovax_idx_feed_id" value="' . esc_attr($id_feed) . '" >
              <input type="hidden" name="wovax_idx_map_state" value="1" >
              <input type="hidden" name="wovax_idx_map_order" value="0" >
              <input type="hidden" name="wovax_idx_map_title" value="Map" >
            </li>
            <li>
              <label class="menu-item-title">
                <input type="checkbox" class="menu-item-checkbox" name="wovax_idx_map_option" value="URL"> Virtual Tour URL
              </label>
              <input type="hidden" name="wovax_idx_feed_id" value="' . esc_attr($id_feed) . '" >
              <input type="hidden" name="wovax_idx_link_state" value="1" >
              <input type="hidden" name="wovax_idx_link_order" value="0" >
              <input type="hidden" name="wovax_idx_link_title" value="Virtual Tour URL" >
            </li>';
    foreach($api_data[0] as $key => $item) {
      $new_value = (!empty($item->field_alias)) ? $item->field_alias : $item->default_alias;
      $old_value = (!empty($item->field_alias)) ? $item->field_alias : $item->default_alias;
      $label_val = (empty($item->default_alias)) ? $item->field_alias : $item->default_alias;
      if ($object = wovax_idx_find_by_name_feed($list_result, $item->id)) {
        if (!empty($object->alias_update)) {
          $new_value = $object->alias_update;
        }
      }
      $txt_value = ($new_value == $item->field_alias || $new_value == $item->default_alias) ? '' : $new_value;

      $container .='<input type="hidden" name="wovax_idx_old_field_' . esc_attr($item->id) . '"  value="' . esc_attr($old_value) . '" >
                    <input type="hidden" name="wovax_idx_id_field_' . esc_attr($item->id) . '" value="' . esc_attr($item->id) . '" >
                    <input type="hidden" name="wovax_idx_field_name_' . esc_attr($item->id) . '" value="' . esc_attr($item->mls_field_name) . '" >
                    <input type="hidden" name="wovax_idx_status_alias_' . esc_attr($item->id) . '" value="' . esc_attr($item->status_alias) . '" >
                    <input type="hidden" name="wovax_idx_default_alias_' . esc_attr($item->id) . '" value="' . esc_attr($item->default_alias) . '" >
                    <input type="hidden" name="wovax_idx_field_alias_' . esc_attr($item->id) . '" value="' . esc_attr($item->field_alias) . '" >';

      $all.='<li>
                  <label class="menu-item-title">
                    <input type="checkbox" class="menu-item-checkbox" name="wovax_idx_id_field_' . esc_attr($item->id) . '" value="' . esc_attr($item->mls_field_name) . '"> ' . esc_html($label_val) . '
                  </label>
                  <input type="hidden" name="wovax_idx_id"  value="' . esc_attr($item->id) . '" >
                  <input type="hidden" name="wovax_idx_old_field"  value="' . esc_attr($old_value) . '" >
                  <input type="hidden" name="wovax_idx_field_name" value="' . esc_attr($item->mls_field_name) . '" >
                </li>';

      if(in_array($label_val, $values_array)){
        $default.='<li>
                  <label class="menu-item-title">
                    <input type="checkbox" class="menu-item-checkbox" name="wovax_idx_id_field_' . esc_attr($item->id) . '" value="' . esc_attr($item->mls_field_name) . '"> ' . esc_html($label_val) . '
                  </label>
                  <input type="hidden" name="wovax_idx_id"  value="' . esc_attr($item->id) . '" >
                  <input type="hidden" name="wovax_idx_old_field"  value="' . esc_attr($old_value) . '" >
                  <input type="hidden" name="wovax_idx_field_name" value="' . esc_attr($item->mls_field_name) . '" >
                </li>';
      }
    }
  }

  echo json_encode(array(
    'all' => $all,
    'default' => $default,
    'container' => $container
  ));
  die();
}

/* function to retrieve all fields into the wovax_IDX_detail form */
function wovax_idx_get_result_api_feed_for_search_content() {
  if (!isset($_POST['attr'])) {
    echo json_encode(array());
    die();
  }

  $id_feed = filter_var($_POST['attr']['idfeed'], FILTER_SANITIZE_STRING);
  $value = filter_var($_POST['attr']['value'], FILTER_SANITIZE_STRING);

  $api_data = wovax_idx_api_data_feed_details($id_feed);

  $search = [];

  $list_result = wovax_idx_list_feeds_field_by_id($id_feed);

  if ($api_data[0][0] == 'No data found') {
    $search = 'No data found';
  }
  else {
    
    foreach($api_data[0] as $key => $item) {
      $old_value = (!empty($item->field_alias)) ? $item->field_alias : $item->default_alias;
      $label_val = (empty($item->default_alias)) ? $item->field_alias : $item->default_alias;

      if(strpos( strtolower($label_val), strtolower($value))!== false){
        $search[$item->id] = '<li>
                  <label class="menu-item-title">
                    <input type="checkbox" class="menu-item-checkbox" name="wovax_idx_id_field_' . esc_attr($item->id) . '" value="' . esc_attr($item->mls_field_name) . '"> ' . esc_html($label_val) . '
                  </label>
                  <input type="hidden" name="wovax_idx_id"  value="' . esc_attr($item->id) . '" >
                  <input type="hidden" name="wovax_idx_old_field"  value="' . esc_attr($old_value) . '" >
                  <input type="hidden" name="wovax_idx_field_name" value="' . esc_attr($item->mls_field_name) . '" >
                </li>';
      }
    }
  }

  echo json_encode(array(
    'search' => $search
  ));
  die();
}

/* function to order the filters or rules */
function wovax_idx_set_order_section() {
  global $wpdb;
  $order_number = (array)json_decode(stripslashes($_POST['order_json']));
  $order_json = json_encode($order_number);

  if ( filter_var($_POST['option'], FILTER_SANITIZE_STRING) == 'feed_rule' ) {
    $id = filter_var($_POST['id_feed'], FILTER_SANITIZE_STRING);
    $feed_attr_info = wovax_idx_get_feed_attr_by_id_feed($id);
    $feed_attr_object = json_decode($feed_attr_info[0]->attributes);
    $feed_attr_object->feed_rules_order = $order_json;

    $feed_attr_object = json_encode($feed_attr_object);
    $sql = "INSERT INTO `{$wpdb->base_prefix}wovax_idx_feeds` (`id_feed`, `attributes`) VALUES (%d, %s) ON DUPLICATE KEY UPDATE `attributes`= %s;";
    $wpdb->query($wpdb->prepare($sql, array($id, $feed_attr_object, $feed_attr_object)));
    exit;  
    
  }else {

    $id = filter_var($_POST['id_shortcode'], FILTER_SANITIZE_STRING);
    /* save data */
    $sql = "UPDATE `{$wpdb->base_prefix}wovax_idx_shortcode` SET `order_section`= %s WHERE `id` = %d ;";
    $wpdb->query($wpdb->prepare($sql, $order_json, $id));
    exit;  

  }
}

/* function to save a favorite property */
function wovax_idx_save_favorite() {
  $fav_array = array();
  $user = wp_get_current_user();
  $propert_json = get_user_meta($user->ID) ['wovax-idx-favorites'][0];
  $new_fav = array( filter_var(stripslashes($_POST['feed_id']) , FILTER_SANITIZE_STRING),
                    filter_var(stripslashes($_POST['mls']) , FILTER_SANITIZE_STRING));
  if ($propert_json != null) {
    $fav_array = (array)json_decode($propert_json);
  }

  if (!in_array($new_fav, $fav_array)) {
    array_push($fav_array, $new_fav);
  }

  $fav_array = json_encode($fav_array);
  update_user_meta($user->ID, 'wovax-idx-favorites', $fav_array);
  exit;
}

/* function to delete a favorite property */
function wovax_idx_delete_favorite() {
  $fav_array = array();
  $user = wp_get_current_user();
  $propert_json = get_user_meta($user->ID) ['wovax-idx-favorites'][0];
  $del_fav = array( filter_var(stripslashes($_POST['feed_id']) , FILTER_SANITIZE_STRING),
                    filter_var(stripslashes($_POST['mls']) , FILTER_SANITIZE_STRING));
  $fav_array = (array)json_decode($propert_json);
  $key = array_search($del_fav, $fav_array);
  unset($fav_array[$key]);
  $fav_array = array_values($fav_array);
  $fav_array = json_encode($fav_array);
  update_user_meta($user->ID, 'wovax-idx-favorites', $fav_array);
  exit;
}

function wovax_idx_update_profile() {
  $id              = filter_var($_POST['id'], FILTER_SANITIZE_STRING) ? filter_var($_POST['id'], FILTER_SANITIZE_STRING):'';
  $firstname       = filter_var($_POST['firstname'], FILTER_SANITIZE_STRING) ? filter_var($_POST['firstname'], FILTER_SANITIZE_STRING):'';
  $lastname        = filter_var($_POST['lastname'], FILTER_SANITIZE_STRING) ? filter_var($_POST['lastname'], FILTER_SANITIZE_STRING):'';
  $email           = filter_var($_POST['email'], FILTER_SANITIZE_STRING) ? filter_var($_POST['email'], FILTER_SANITIZE_STRING):'';
  $password        = filter_var($_POST['password'], FILTER_SANITIZE_STRING) ? filter_var($_POST['password'], FILTER_SANITIZE_STRING):'';
  $verify_password = filter_var($_POST['verify_password'], FILTER_SANITIZE_STRING) ? filter_var($_POST['verify_password'], FILTER_SANITIZE_STRING):'';
  $phone           = filter_var($_POST['phone'], FILTER_SANITIZE_STRING) ? filter_var($_POST['phone'], FILTER_SANITIZE_STRING):'';
  $username        = filter_var($_POST['username'], FILTER_SANITIZE_STRING) ? filter_var($_POST['username'], FILTER_SANITIZE_STRING):'';

  $update_values = array( 'ID' => $id);
  if($firstname != null && $firstname != ''){$update_values['first_name'] = $firstname;}
  if($lastname != null && $lastname != ''){$update_values['last_name'] = $lastname;}
  if($email != null && $email != ''){$update_values['user_email'] = $email;}
  if($password != null && $password != '' && $verify_password != null && $verify_password != '' && $password == $verify_password){$update_values['user_pass'] = $password;}
  if($username != null && $username != ''){$update_values['user_login'] = $username;}

  if($phone != null && $phone != ''){update_user_meta($id, 'phone', $phone);}

  $update_user = wp_update_user( $update_values );


  if ( is_wp_error( $update_user ) ) {
    $result = 'There was an error';
  } else {
    $result = 'Success!';
  }

  echo json_encode(array(
    'value' => $result
  ));

  exit();
}
