<?php
/*
Plugin Name: Wovax IDX
Plugin URI: https://wovax.com/idx/
Description: Wovax IDX is a WordPress plugin built for realtors & developers. Wovax IDX allows you to easily display MLS real estate listings on your WordPress website.
Version: 0.5.3
Author: Wovax, LLC.
Author URI: https://wovax.com/
License: GPLv2 or later
*/

// Plugin admin menu

if (!defined('ABSPATH')) exit;
// Include files
include_once(__DIR__.'/admin/pages/search-appearance.php');
require_once(__DIR__.'/api/connect.php');
require_once(__DIR__.'/settings/feed-fields.php');
require_once(__DIR__.'/settings/options.php');
require_once(__DIR__.'/settings/initial-setup.php');
require_once(__DIR__.'/settings/search-appearance.php');
require_once(__DIR__.'/shortcodes/shortcode.php');
require_once(__DIR__.'/shortcodes/listing-details.php');
require_once(__DIR__.'/utilities/listing-loader.php');

add_action('admin_menu', 'wovax_idx_menu');
require plugin_dir_path(__FILE__) . "/functions.php";

require plugin_dir_path(__FILE__) . "/wovax-idx-ajax.php";

require plugin_dir_path(__FILE__) . "/shortcodes/wovax-idx-shortcode.php";

if (!defined('WOVAX_PLUGIN_NAME')) {
  define('WOVAX_PLUGIN_NAME', plugins_url('/', __FILE__) . '/');
}

if ( ! class_exists( 'WP_List_Table' ) ) {
  require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Feeds_List extends WP_List_Table {

  function get_columns() {
    $columns = [
      'resource'    => __( 'Resource', 'sp' ),
      'feed'        => __( 'Feed', 'sp' ),
      'board'       => __( 'Board', 'sp' ),
      'environment' => __( 'Environment', 'sp' ),
      'status'      => __( 'Status', 'sp' ),
      'updated'     => __( 'Updated', 'sp' )
    ];

    return $columns;
  }
}

class Shortcode_List extends WP_List_Table {

  function get_columns() {
    $columns = [
      'shortcode' => __( 'Shortcode', 'sp' ),
      'type'      => __( 'Type', 'sp' ),
      'author'    => __( 'Author', 'sp' ),
      'created'   => __( 'Created', 'sp' )
    ];

    return $columns;
  }
}

class User_List extends WP_List_Table {

  function get_columns() {
    $columns = [
      'fullname'  => __( 'Fullname', 'sp' ),
      'phone'     => __( 'Phone', 'sp' ),
      'email'     => __( 'Email', 'sp' ),
      'favorites' => __( 'Favorites', 'sp' )
    ];

    return $columns;
  }
}

function wovax_idx_menu() {
  // $page_title, $menu_title, $capability, $menu_slug, $function = '', $icon_url = '', $position = null
  add_menu_page('Wovax IDX', 'Wovax IDX', 'manage_options', 'wovax_idx_settings', '', 'dashicons-admin-multisite', '25.00000000000001');
  // $parent_slug, $page_title, $menu_title, $capability, $menu_slug, $function = ''
  $settngs_hook = add_submenu_page('wovax_idx_settings', 'Wovax IDX Settings', 'Settings', 'manage_options', 'wovax_idx_settings', 'wovax_idx_settings');
  $feeds_hook   = add_submenu_page('wovax_idx_settings', 'Wovax IDX Feeds', 'IDX Feeds', 'manage_options', 'wovax_idx_feeds', 'wovax_idx_feeds');
  $short_hook   = add_submenu_page('wovax_idx_settings', 'Wovax IDX Shortcodes', 'Shortcodes', 'manage_options', 'wovax_idx_shortcodes', 'wovax_idx_shortcodes');
  $user_hook    = add_submenu_page('wovax_idx_settings', 'Wovax IDX User Activity', 'User Activity', 'manage_options', 'wovax_idx_user_activity', 'wovax_idx_user_activity');
}

// Initial Setup admin page
function wovax_idx_settings() {
  require plugin_dir_path(__FILE__) . "/admin/pages/settings.php";
}

// IDX Feeds admin page
function wovax_idx_feeds() {
  if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'update'):
    require plugin_dir_path(__FILE__) . "/admin/pages/idx-feeds-details.php";
  else:
    require plugin_dir_path(__FILE__) . "/admin/pages/idx-feeds.php";
  endif;
}

// Shortcodes admin page
function wovax_idx_shortcodes() {
  if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'update'):
    require plugin_dir_path(__FILE__) . "/admin/pages/shortcodes-details.php";
  else:
    require plugin_dir_path(__FILE__) . "/admin/pages/shortcodes.php";
  endif;
}

function wovax_idx_user_activity() {
  if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'update'):
    require plugin_dir_path(__FILE__) . "/admin/pages/user-activity-details.php";
  else:
    require plugin_dir_path(__FILE__) . "/admin/pages/user-activity.php";

  endif;
}

// WordPress Plugins page Settings action link
add_filter('plugin_action_links_' . plugin_basename(__FILE__) , 'wovax_idx_add_action_links');

function wovax_idx_add_action_links($links) {
  $mylinks = array(
    '<a href="' . admin_url('admin.php?page=wovax_idx_settings') . '">Settings</a>',
  );
  return array_merge($mylinks, $links);
}

// Scripts & Styles Public
function wovax_idx_scripts_styles() {
  if (get_option('wovax-idx-settings-styling-floating-label') == 'yes') {
    wp_enqueue_script('wovax_idx_script_float', plugins_url('assets/libraries/float-labels/float-labels.min.js', __FILE__) , array(
      'jquery'
    ));
    wp_enqueue_style('wovax_idx_min_stylesheet_float', plugins_url('assets/libraries/float-labels/float-labels.min.css', __FILE__));
  }

  if (get_option('wovax-idx-settings-styling-css') == 'yes') {
    wp_enqueue_style('wovax_idx_min_stylesheet', plugins_url('assets/css/wovax-idx.min.css', __FILE__));
  }

  if (get_option('wovax-idx-settings-styling-js') == 'yes') {
    wp_enqueue_script('wovax_idx_script', plugins_url('assets/js/wovax-idx.min.js', __FILE__) , array(
      'jquery'
    ));
  }
  wp_enqueue_script('jquery');
}

add_action('wp_enqueue_scripts', 'wovax_idx_scripts_styles');

function wovax_idx_add_styles() {
  wp_enqueue_style('wovax_idx_best_css', plugins_url('assets/libraries/jquery-modal/jquery.modal.min.css', __FILE__));
}

add_action('wp_enqueue_scripts', 'wovax_idx_add_styles');

function wovax_idx_add_scripts() {
  wp_enqueue_script('wovax_idx_best_js', plugins_url('assets/libraries/jquery-modal/jquery.modal.min.js', __FILE__) , array(
    'jquery'
  ));
  wp_enqueue_script('jquery');
}

add_action('wp_enqueue_scripts', 'wovax_idx_add_scripts');

// Scripts & Styles Admin
function wovax_idx_scripts_admin($hook) {
  $variables_array = array(
    'ajax_url' => admin_url('admin-ajax.php') ,
    'paged' => (isset($_REQUEST['paged'])) ? filter_var($_REQUEST['paged'], FILTER_SANITIZE_STRING) : 1,
    's' => (isset($_REQUEST['s'])) ? filter_var($_REQUEST['s'], FILTER_SANITIZE_STRING) : '',
    'orderby' => (isset($_REQUEST['orderby'])) ? filter_var($_REQUEST['orderby'], FILTER_SANITIZE_STRING) : 'ASC',
    'ordername' => (isset($_REQUEST['ordername'])) ? filter_var($_REQUEST['ordername'], FILTER_SANITIZE_STRING) : 'feed_id',
    'filter' => (isset($_REQUEST['filter'])) ? filter_var($_REQUEST['filter'], FILTER_SANITIZE_STRING) : '',
    'idfeed' => (isset($_REQUEST['idfeed'])) ? filter_var($_REQUEST['idfeed'], FILTER_SANITIZE_STRING) : '',
  );
  $variables_array["type_page"] = '';

  if (strpos($hook, 'wovax_idx_settings') !== false) {
    $variables_array["type_page"] = 'wovax_idx_settings';
  }
  elseif (strpos($hook, 'wovax_idx_feeds') !== false && !isset($_REQUEST['action'])) {
    $variables_array["type_page"] = 'wovax_idx_feeds_list';
  }
  elseif (strpos($hook, 'wovax_idx_feeds') !== false && isset($_REQUEST['action']) == 'updated') {
    $variables_array["type_page"] = 'wovax_idx_feeds_detail';
  }
  elseif (strpos($hook, 'wovax_idx_shortcodes') !== false && !isset($_REQUEST['action'])) {
    $variables_array["type_page"] = 'wovax_idx_shortcodes';
  }
  elseif (strpos($hook, 'wovax_idx_user_activity') !== false && !isset($_REQUEST['action'])) {
    $variables_array["type_page"] = 'wovax_idx_user_activity';
  }

  wp_enqueue_script('jquery-ui-sortable', array(
    'jquery'
  ));
  wp_register_script('wovax-idx-feed-js', WOVAX_PLUGIN_NAME . 'admin/assets/js/wovax-idx-feed-general.js', array(
    'jquery'
  ) , '1.0', true);
  wp_localize_script('wovax-idx-feed-js', 'object_name', $variables_array);
  wp_enqueue_script('wovax-idx-feed-js');
  wp_enqueue_style('wovax_idx_admin', plugins_url('admin/assets/css/wovax-idx-admin.css', __FILE__));
}

add_action('admin_enqueue_scripts', 'wovax_idx_scripts_admin');

function wovax_idx_display_options() {
  // initial_setup fields
  register_setting("settings_menu_initial_setup", "wovax-idx-settings-webmaster-email");
  register_setting("settings_menu_initial_setup", "wovax-idx-settings-environment");
  register_setting("settings_menu_initial_setup", "wovax-idx-settings-search-results-page");
  register_setting("settings_menu_initial_setup", "wovax-idx-settings-listing-details-page");
  register_setting("settings_menu_initial_setup", "wovax-idx-settings-google-maps-api-key");
  // listings users
  register_setting("settings_menu_users", "wovax-idx-settings-users-registration");
  register_setting("settings_menu_users", "wovax-idx-settings-users-registration-force");
  register_setting("settings_menu_users", "wovax-idx-settings-users-registration-force-count");
  register_setting("settings_menu_users", "wovax-idx-settings-users-favorites");
  register_setting("settings_menu_users", "wovax-idx-settings-users-saved-searches");
  // listings fields
  register_setting("settings_menu_listing", "wovax-idx-settings-styling-css");
  register_setting("settings_menu_listing", "wovax-idx-settings-styling-js");
  register_setting("settings_menu_listing", "wovax-idx-settings-styling-floating-label");
}

add_action('admin_init', 'wovax_idx_display_options');

// function to create the DB / Options / Defaults
function wovax_idx_options_install() {
  add_option('wovax-idx-settings-styling-css', 'yes');
  add_option('wovax-idx-settings-styling-js', 'yes');
  add_option('wovax-idx-settings-styling-floating-label', 'yes');
  add_option('wovax-idx-settings-users-registration', 'yes');
  add_option('wovax-idx-settings-users-registration-force', '');
  add_option('wovax-idx-settings-users-registration-force-count', '10');
  add_option('wovax-idx-settings-users-favorites', 'yes');
  add_option('wovax-idx-settings-users-saved-searches', 'yes');
  add_option('wovax-idx-feeds-per-page', '10');
  wovax_idx_create_table_feed_fields();
  wovax_idx_create_table_feed();
  wovax_idx_create_table_feed_rules();
  wovax_idx_create_table_shortcode();
  wovax_idx_create_table_shortcode_filters();
  wovax_idx_create_table_shortcode_rules();
  flush_rewrite_rules();
}

function wovax_idx_create_table_feed_fields() {
  global $wpdb;
  $sql_create_fields = "
    CREATE TABLE IF NOT EXISTS `{$wpdb->base_prefix}wovax_idx_feed_fields` (
      `id_feed` int(11) DEFAULT NULL,
      `id_field` int(11) DEFAULT NULL,
      `name` varchar(255) DEFAULT NULL,
      `alias_old` varchar(255) DEFAULT NULL,
      `alias_update` varchar(255) DEFAULT NULL,
      `status_alias` int(11) DEFAULT NULL,
      `default_alias` varchar(255) DEFAULT NULL,
      `field_alias` varchar(255) DEFAULT NULL,
      `field_state` int(1) DEFAULT 0,
      `order` int DEFAULT 0
    ) ENGINE=MyISAM DEFAULT CHARSET=utf8;";
  $wpdb->query($sql_create_fields);
  if($wpdb->last_error !== '') {
      error_log($wpdb->last_error);
  }
}

function wovax_idx_create_table_feed() {
  global $wpdb;
  $sql_create_feed = "
    CREATE TABLE IF NOT EXISTS `{$wpdb->base_prefix}wovax_idx_feeds` (
      `id_feed` int(11) NOT NULL PRIMARY KEY,
      `attributes` longtext DEFAULT NULL
    ) ENGINE=MyISAM DEFAULT CHARSET=utf8;";
  $wpdb->query($sql_create_feed);
  if($wpdb->last_error !== '') {
      error_log($wpdb->last_error);
  }
}

function wovax_idx_create_table_feed_rules() {
  global $wpdb;
  $sql_create_feed_rules = "
    CREATE TABLE IF NOT EXISTS `{$wpdb->base_prefix}wovax_idx_feed_rules` (
      `id_rule` int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
      `id_feed` int(11) DEFAULT NULL,
      `id_field` varchar(255) DEFAULT NULL,
      `field` varchar(255) DEFAULT NULL,
      `rule_type` varchar(255) DEFAULT NULL,
      `rule_value` varchar(255) DEFAULT NULL,
      `date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP
    ) ENGINE=MyISAM DEFAULT CHARSET=utf8;";
  $wpdb->query($sql_create_feed_rules);
  if($wpdb->last_error !== '') {
      error_log($wpdb->last_error);
  }
}

function wovax_idx_create_table_shortcode() {
  global $wpdb;
  $sql_create_shortcode = "
    CREATE TABLE IF NOT EXISTS `{$wpdb->base_prefix}wovax_idx_shortcode` (
      `id` int(11)  NOT NULL AUTO_INCREMENT PRIMARY KEY,
      `type` varchar(255) DEFAULT NULL,
      `title` varchar(255) DEFAULT NULL,
      `grid_view` varchar(255) DEFAULT NULL,
      `map_view` varchar(255) DEFAULT NULL,
      `per_page` varchar(255) DEFAULT NULL,
      `per_map` varchar(255) DEFAULT NULL,
      `author` varchar(255) DEFAULT NULL,
      `date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
      `status` varchar(255) DEFAULT NULL,
      `pagination` varchar(255) DEFAULT NULL,
      `feeds` longtext DEFAULT NULL,
      `order_section` text DEFAULT NULL
    ) ENGINE=MyISAM DEFAULT CHARSET=utf8;";
  $wpdb->query($sql_create_shortcode);
  if($wpdb->last_error !== '') {
      error_log($wpdb->last_error);
  }
}

function wovax_idx_create_table_shortcode_filters() {
  global $wpdb;
  $sql_create_filters = "
    CREATE TABLE IF NOT EXISTS `{$wpdb->base_prefix}wovax_idx_shortcode_filters` (
      `id_filter` int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
      `id_shortcode` int(11) DEFAULT NULL,
      `id_field` varchar(255) DEFAULT NULL,
      `field` varchar(255) DEFAULT NULL,
      `filter_type` varchar(255) DEFAULT NULL,
      `filter_label` varchar(255) DEFAULT NULL,
      `filter_placeholder` varchar(255) DEFAULT NULL,
      `date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP
    ) ENGINE=MyISAM DEFAULT CHARSET=utf8;";
  $wpdb->query($sql_create_filters);
  if($wpdb->last_error !== '') {
      error_log($wpdb->last_error);
  }
}

function wovax_idx_create_table_shortcode_rules() {
  global $wpdb;
  $sql_create_shortcode_rules = "
    CREATE TABLE IF NOT EXISTS `{$wpdb->base_prefix}wovax_idx_shortcode_rules` (
      `id_rule` int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
      `id_shortcode` int(11) DEFAULT NULL,
      `id_field` varchar(255) DEFAULT NULL,
      `field` varchar(255) DEFAULT NULL,
      `rule_type` varchar(255) DEFAULT NULL,
      `rule_value` varchar(255) DEFAULT NULL,
      `date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP
    ) ENGINE=MyISAM DEFAULT CHARSET=utf8;";
  $wpdb->query($sql_create_shortcode_rules);
  if($wpdb->last_error !== '') {
      error_log($wpdb->last_error);
  }
}

// run the install scripts upon plugin activation
register_activation_hook(__FILE__, 'wovax_idx_options_install');
register_deactivation_hook(__FILE__, function() {
	flush_rewrite_rules();
});

add_action('admin_init', 'wovax_idx_init_shortcode_post');
add_action('admin_init', 'wovax_idx_init_shortcode_get');
add_action('admin_init', 'wovax_idx_init_feed_post');

function wovax_idx_init_shortcode_post() {
  if ($_SERVER['REQUEST_METHOD'] === 'POST' && $_GET['page'] == 'wovax_idx_shortcodes' && $_GET['action'] == 'update') {
    if (isset($_POST['wovax_idx_shortcode_fields']) && wp_verify_nonce(filter_var($_POST['wovax_idx_shortcode_fields'], FILTER_SANITIZE_STRING) , 'save_change_shortcode_fields')) {
      $id_shrotcode = wovax_idx_save_shortcode(filter_var_array($_POST, FILTER_SANITIZE_STRING));
      $url = admin_url("admin.php?page=wovax_idx_shortcodes&tab=general&id=$id_shrotcode&action=update");
      wp_redirect($url);
    }
    elseif (isset($_POST['wovax_idx_shortcode_view']) && wp_verify_nonce(filter_var($_POST['wovax_idx_shortcode_view'], FILTER_SANITIZE_STRING) , 'save_change_shortcode_view')) {
      $id_shrotcode = wovax_idx_save_shortcode_view(filter_var_array($_POST, FILTER_SANITIZE_STRING));
      $url = admin_url("admin.php?page=wovax_idx_shortcodes&tab=view&id=$id_shrotcode&action=update");
      wp_redirect($url);
    }
    elseif (isset($_POST['wovax_idx_shortcode_feeds']) && wp_verify_nonce(filter_var($_POST['wovax_idx_shortcode_feeds'], FILTER_SANITIZE_STRING) , 'save_change_shortcode_feeds')) {
      $id_shrotcode = wovax_idx_save_shortcode_feeds(filter_var_array($_POST, FILTER_SANITIZE_STRING));
      $url = admin_url("admin.php?page=wovax_idx_shortcodes&tab=feeds&id=$id_shrotcode&action=update");
      wp_redirect($url);
    }
    elseif (isset($_POST['wovax_idx_shortcode_filters']) && wp_verify_nonce(filter_var($_POST['wovax_idx_shortcode_filters'], FILTER_SANITIZE_STRING) , 'save_change_shortcode_filters') && isset($_POST['submit']) && $_POST['submit'] == 'Add Filter' || $_POST['submit'] == 'Edit Filter') {
      $id_shrotcode = wovax_idx_save_shortcode_filters(filter_var_array($_POST, FILTER_SANITIZE_STRING));
      $url = admin_url("admin.php?page=wovax_idx_shortcodes&tab=filters&id=$id_shrotcode&action=update");
      wp_redirect($url);
    }
    elseif (isset($_POST['wovax_idx_shortcode_rules']) && wp_verify_nonce(filter_var($_POST['wovax_idx_shortcode_rules'], FILTER_SANITIZE_STRING) , 'save_change_shortcode_rules') && isset($_POST['submit']) && $_POST['submit'] == 'Add Rule' || $_POST['submit'] == 'Edit Rule') {
      $id_shrotcode = wovax_idx_save_shortcode_rules(filter_var_array($_POST, FILTER_SANITIZE_STRING));
      $url = admin_url("admin.php?page=wovax_idx_shortcodes&tab=rules&id=$id_shrotcode&action=update");
      wp_redirect($url);
    }
  }
}

// function that duplicate the shortcode
function wovax_idx_init_shortcode_get() {
  if (isset($_GET['action'], $_GET['page'], $_GET['id']) && $_GET['action'] == 'duplicate' && $_GET['page'] == 'wovax_idx_shortcodes' && !isset($_GET['idfilter'])) {
    $id_shortcode = filter_var($_GET['id'], FILTER_SANITIZE_STRING);
    wovax_idx_shortcode_duplicate_table($id_shortcode);
    $url = admin_url("admin.php?page=wovax_idx_shortcodes");
    wp_redirect($url);
  }
  elseif (isset($_GET['action'], $_GET['page'], $_GET['id'], $_GET['idfilter']) && $_GET['action'] == 'duplicate' && $_GET['page'] == 'wovax_idx_shortcodes') {
    $id_shortcode = filter_var($_GET['id'], FILTER_SANITIZE_STRING);
    $id_filter = filter_var($_GET['idfilter'], FILTER_SANITIZE_STRING);
    wovax_idx_shortcode_duplicate_filter($id_shortcode, $id_filter);
    $url = admin_url("admin.php?page=wovax_idx_shortcodes&action=update&id=$id_shortcode&tab=filters");
    wp_redirect($url);
  }
}

// function that duplicate the shortcode
function wovax_idx_init_feed_post() {
  if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['wovax_idx_feeds_feed_fields']) && wp_verify_nonce($_POST['wovax_idx_feeds_feed_fields'], 'save_change_feeds_fields')) {
      /* function to store all fields that belong to the form on Fields tab */
      wovax_idx_save_fields(filter_var_array($_POST, FILTER_SANITIZE_STRING));
    }
  }

  if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['wovax_idx_feeds_field_states']) && wp_verify_nonce($_POST['wovax_idx_feeds_field_states'], 'save_change_field_states')) {
      /* function to store update all field_state that belong to the form on Fields Layout */
      wovax_idx_save_field_states(filter_var_array($_POST, FILTER_SANITIZE_STRING));
    }
  }

  if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['wovax_idx_feeds_feed_general']) && wp_verify_nonce($_POST['wovax_idx_feeds_feed_general'], 'save_change_feeds_general')) {
      /* function to store all fields that belong to the form on Fields tab */
      wovax_idx_save_feed_general(filter_var_array($_POST, FILTER_SANITIZE_STRING));
    }
  }

  if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['wovax_idx_feeds_feed_rules']) && wp_verify_nonce($_POST['wovax_idx_feeds_feed_rules'], 'save_change_feeds_rules')) {
      /* function to store all rules that belong to the form on Fields tab */
      $id_feed = wovax_idx_save_feed_rules(filter_var_array($_POST, FILTER_SANITIZE_STRING));
      $url = admin_url("admin.php?page=wovax_idx_feeds&tab=rules&action=update&idfeed=$id_feed");
      wp_redirect($url);
    }
  }
}

add_action( "load-wovax-idx_page_wovax_idx_feeds", 'wovax_idx_feeds_options' );
add_action( "load-wovax-idx_page_wovax_idx_shortcodes", 'wovax_idx_shortcodes_options' );
add_action( "load-wovax-idx_page_wovax_idx_user_activity", 'wovax_idx_users_options' );

function wovax_idx_feeds_options() {

  if(!isset($_GET['tab'])){

    $feed_per_page = get_option("wovax-idx-feeds-per-page");
    $option = 'per_page';

    $args = array(
        'label' => 'Number of feeds per page',
        'default' => $feed_per_page,
        'option' => 'wovax-idx-feeds-per-page'
    );

    add_screen_option( $option, $args );
    $feed_list = new Feeds_List();

  }
}

function wovax_idx_shortcodes_options() {

  if(!isset($_GET['action'])){

    $short_per_page = get_option("wovax-idx-shortcodes-per-page");
    $option = 'per_page';

    $args = array(
        'label' => 'Number of shortcodes per page',
        'default' => $short_per_page,
        'option' => 'wovax-idx-shortcodes-per-page'
    );

    add_screen_option( $option, $args );
    $shortcode_list = new Shortcode_List();

  }
}

function wovax_idx_users_options() {

  if(!isset($_GET['action'])){

    $short_per_page = get_option("wovax-idx-users-per-page");
    $option = 'per_page';

    $args = array(
        'label' => 'Number of users per page',
        'default' => $short_per_page,
        'option' => 'wovax-idx-users-per-page'
    );

    add_screen_option( $option, $args );
    $user_list = new User_List();

  }
}

add_filter('set-screen-option', 'wovax_idx_set_option', 10, 3);

function wovax_idx_set_option($status, $option, $value) {

    if ( 'wovax_idx_feeds_per_page' == $option ) {
      update_option('wovax-idx-feeds-per-page', $value);
    }elseif ( 'wovax_idx_users_per_page' == $option ) {
      update_option('wovax-idx-users-per-page', $value);
    }else{
      update_option('wovax-idx-shortcodes-per-page', $value);
    }

    return $value;

}
